@@ -5,14 +5,14 @@
  *
  *      Description:  Header File of likwid API
  *
- *      Version:   5.0
- *      Released:  10.11.2019
+ *      Version:   5.2.2
+ *      Released:  26.07.2022
  *
  *      Authors:  Thomas Gruber (tr), thomas.roehl@googlemail.com
  *
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2022 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -83,9 +83,13 @@ of LIKWID. Before you can call likwid_markerThreadInit() you have to call likwid
 
 */
 extern void likwid_markerThreadInit(void) __attribute__ ((visibility ("default") ));
-/*! \brief Select next group to measure
+/*! \brief Switch to next group to measure
 
-Must be called in parallel region of the application to switch group on every CPU.
+Should be called in a serial region of code. If it is to be called from inside
+a parallel region, ensure only one thread runs it by using "#pragma omp single"
+or similar. Additionally, if this function is called in a parallel region,
+ensure that the serial regions is preceeded by a barrier ("#pragma omp barrier"
+or similar) to prevent race conditions. 
 */
 extern void likwid_markerNextGroup(void) __attribute__ ((visibility ("default") ));
 /*! \brief Close LIKWID's marker API
@@ -103,16 +107,27 @@ Initializes the hashTable entry in order to reduce execution time of likwid_mark
 extern int likwid_markerRegisterRegion(const char* regionTag) __attribute__ ((visibility ("default") ));
 /*! \brief Start a measurement region
 
-Reads the values of all configured counters and saves the results under the name given
-in regionTag.
+Reads the values of all configured counters and saves the results under the
+name given in regionTag. Must be called on every thread that is to be measured,
+e.g. if the code to be measured is run in a parallel region, this function must
+also be called in a parallel region (typically the same parallel region as the
+measured code). If this function is to be called multiple times in one parallel
+region, place a barrier ("#pragma omp barrier" or similar) before each call to
+likwid_markerStartRegion
 @param regionTag [in] Store data using this string
 @return Error code of start operation
 */
 extern int likwid_markerStartRegion(const char* regionTag) __attribute__ ((visibility ("default") ));
 /*! \brief Stop a measurement region
 
-Reads the values of all configured counters and saves the results under the name given
-in regionTag. The measurement data of the stopped region gets summed up in global region counters.
+Reads the values of all configured counters and saves the results under the
+name given in regionTag. The measurement data of the stopped region gets summed
+up in global region counters. Must be called on every thread that is to be
+measured, e.g. if the code to be measured is run in a parallel region, this
+function must also be called in a parallel region (typically the same parallel
+region as the measured code). If this function is called multiple times in one
+parallel region, place a barrier ("#pragma omp barrier" or similar) after each
+call to likwid_markerStopRegion
 @param regionTag [in] Store data using this string
 @return Error code of stop operation
 */
@@ -190,7 +205,7 @@ extern void HPMmode(int mode) __attribute__ ((visibility ("default") ));
 /*! \brief Initialize access module
 
 Initialize the module internals to either the MSR/PCI files or the access daemon
-@return error code (0 for sccess)
+@return error code (0 for access)
 */
 extern int HPMinit() __attribute__ ((visibility ("default") ));
 /*! \brief Add CPU to access module
@@ -299,9 +314,9 @@ typedef struct {
     int         supportClientmem; /*!< \brief Flag if system has mappable memory controllers */
     uint64_t    featureFlags; /*!< \brief Mask of all features supported by the CPU*/
     uint32_t    perf_version; /*!< \brief Version of Intel's performance monitoring facility */
-    uint32_t    perf_num_ctr; /*!< \brief Number of general purpose core-local performance monitoring counters */
+    uint32_t    perf_num_ctr; /*!< \brief Number of general purpose HWthread-local performance monitoring counters */
     uint32_t    perf_width_ctr; /*!< \brief Bit width of fixed and general purpose counters */
-    uint32_t    perf_num_fixed_ctr; /*!< \brief Number of fixed purpose core-local performance monitoring counters */
+    uint32_t    perf_num_fixed_ctr; /*!< \brief Number of fixed purpose HWthread-local performance monitoring counters */
 } CpuInfo;
 
 /*! \brief Structure with IDs of a HW thread
@@ -315,7 +330,8 @@ typedef struct {
     uint32_t coreId; /*!< \brief ID of CPU core that executes the HW thread */
     uint32_t packageId; /*!< \brief ID of CPU socket containing the HW thread */
     uint32_t apicId; /*!< \brief ID of HW thread retrieved through the Advanced Programmable Interrupt Controller */
-    uint32_t inCpuSet; /*!< \brief ID of HW thread inside the CPU core */
+    uint32_t dieId; /*!< \brief ID of die. A package might contain multiple dies */
+    uint32_t inCpuSet; /*!< \brief Flag if HW thread is inside the CPUset */
 } HWThread;
 
 /*! \brief Enum of possible caches
@@ -353,9 +369,10 @@ typedef struct {
 This structure describes the topology at HW thread level like the amount of HW threads, how they are distributed over the CPU sockets/packages and how the caching hierarchy is assembled.
 */
 typedef struct {
-    uint32_t numHWThreads; /*!< \brief Amount of HW threads in the system and length of \a threadPool */
+    uint32_t numHWThreads; /*!< \brief Amount of active HW threads in the system (e.g. in cpuset) */
     uint32_t activeHWThreads; /*!< \brief Amount of HW threads in the system and length of \a threadPool */
     uint32_t numSockets; /*!< \brief Amount of CPU sockets/packages in the system */
+    uint32_t numDies; /*!< \brief Amount of CPU dies in the system */
     uint32_t numCoresPerSocket; /*!< \brief Amount of physical cores in one CPU socket/package */
     uint32_t numThreadsPerCore; /*!< \brief Amount of HW threads in one physical CPU core */
     uint32_t numCacheLevels; /*!< \brief Amount of caches for each HW thread and length of \a cacheLevels */
@@ -511,7 +528,7 @@ The AffinityDomain data structure describes a single domain in the current syste
 typedef struct {
     bstring tag; /*!< \brief Bstring with the ID for the affinity domain. Currently possible values: N (node), SX (socket/package X), CX (LLC cache domain X) and MX (memory domain X) */
     uint32_t numberOfProcessors; /*!< \brief Number of HW threads in the domain and length of \a processorList */
-    uint32_t numberOfCores; /*!< \brief Number of CPU cores in the domain */
+    uint32_t numberOfCores; /*!< \brief Number of hardware threads in the domain */
     int*  processorList; /*!< \brief List of HW thread IDs in the domain */
 } AffinityDomain;
 
@@ -529,8 +546,8 @@ typedef struct {
     uint32_t numberOfNumaDomains; /*!< \brief Number of NUMA nodes in the system */
     uint32_t numberOfProcessorsPerSocket; /*!< \brief Number of HW threads per socket/package in the system */
     uint32_t numberOfCacheDomains; /*!< \brief Number of LLC caches in the system */
-    uint32_t numberOfCoresPerCache; /*!< \brief Number of HW threads per LLC cache in the system */
-    uint32_t numberOfProcessorsPerCache; /*!< \brief Number of CPU cores per LLC cache in the system */
+    uint32_t numberOfCoresPerCache; /*!< \brief Number of CPU cores per LLC cache in the system */
+    uint32_t numberOfProcessorsPerCache; /*!< \brief Number of hardware threads per LLC cache in the system */
     uint32_t numberOfAffinityDomains; /*!< \brief Number of affinity domains in the current system  and length of \a domains array */
     AffinityDomain* domains; /*!< \brief List of all domains in the system */
 } AffinityDomains;
@@ -675,7 +692,10 @@ extern void perfmon_returnGroups(int nrgroups, char** groups, char** shortinfos,
 /*! \brief Initialize performance monitoring facility
 
 Initialize the performance monitoring feature by creating basic data structures.
-The access mode must already be set when calling perfmon_init()
+The CPU ids for the threadsToCpu list can be found in cpuTopology->threadPool[thread_id]->apicId.
+The access mode must already be set when calling perfmon_init().
+\sa HPMmode() function and CpuTopology structure with HWThread list
+
 @param [in] nrThreads Amount of threads
 @param [in] threadsToCpu List of CPUs
 @return error code (0 on success, -ERRORCODE on failure)
@@ -684,7 +704,7 @@ extern int perfmon_init(int nrThreads, const int* threadsToCpu) __attribute__ ((
 
 /*! \brief Initialize performance monitoring maps
 
-Initialize the performance monitoring maps for counters, events and Uncore boxes#
+Initialize the performance monitoring maps for counters, events and Uncore boxes
 for the current architecture. topology_init() and numa_init() must be called before calling
 perfmon_init_maps()
 \sa RegisterMap list, PerfmonEvent list and BoxMap list
@@ -1636,98 +1656,105 @@ Initialize cpu frequency module
 @return returns 0 if successfull and 1 if invalid accessmode
 */
 extern int freq_init(void) __attribute__ ((visibility ("default") ));
-/*! \brief Get the current clock frequency of a core
+/*! \brief Get the base clock frequency of a hardware thread
 
-Get the current clock frequency of a core
+Get the base clock frequency of a hardware thread
+@param [in] cpu_id CPU ID
+@return Frequency or 0 in case of errors
+*/
+uint64_t freq_getCpuClockBase(const int cpu_id) __attribute__ ((visibility ("default") ));
+/*! \brief Get the current clock frequency of a hardware thread
+
+Get the current clock frequency of a hardware thread
 @param [in] cpu_id CPU ID
 @return Frequency or 0 in case of errors
 */
 extern uint64_t freq_getCpuClockCurrent(const int cpu_id ) __attribute__ ((visibility ("default") ));
 
-/*! \brief Get the maximal clock frequency of a core
+/*! \brief Get the maximal clock frequency of a hardware thread
 
-Get the maximal clock frequency of a core
+Get the maximal clock frequency of a hardware thread
 @param [in] cpu_id CPU ID
 @return Frequency or 0 in case of errors
 */
 extern uint64_t freq_getCpuClockMax(const int cpu_id ) __attribute__ ((visibility ("default") ));
-/*! \brief Get the maximal available clock frequency of a core
+/*! \brief Get the maximal available clock frequency of a hardware thread
 
-Get the maximal clock frequency of a core
+Get the maximal clock frequency of a hardware thread
 @param [in] cpu_id CPU ID
 @return Frequency or 0 in case of errors
 */
 extern uint64_t freq_getConfCpuClockMax(const int cpu_id) __attribute__ ((visibility ("default") ));
-/*! \brief Set the maximal clock frequency of a core
+/*! \brief Set the maximal clock frequency of a hardware thread
 
-Set the maximal clock frequency of a core
+Set the maximal clock frequency of a hardware thread
 @param [in] cpu_id CPU ID
 @param [in] freq Frequency in kHz
 @return Frequency or 0 in case of errors
 */
 extern uint64_t freq_setCpuClockMax(const int cpu_id, const uint64_t freq) __attribute__ ((visibility ("default") ));
-/*! \brief Get the minimal clock frequency of a core
+/*! \brief Get the minimal clock frequency of a hardware thread
 
-Get the minimal clock frequency of a core
+Get the minimal clock frequency of a hardware thread
 @param [in] cpu_id CPU ID
 @return Frequency or 0 in case of errors
 */
 extern uint64_t freq_getCpuClockMin(const int cpu_id ) __attribute__ ((visibility ("default") ));
-/*! \brief Get the minimal available clock frequency of a core
+/*! \brief Get the minimal available clock frequency of a hardware thread
 
-Get the minimal clock frequency of a core
+Get the minimal clock frequency of a hardware thread
 @param [in] cpu_id CPU ID
 @return Frequency or 0 in case of errors
 */
 extern uint64_t freq_getConfCpuClockMin(const int cpu_id) __attribute__ ((visibility ("default") ));
-/*! \brief Set the minimal clock frequency of a core
+/*! \brief Set the minimal clock frequency of a hardware thread
 
-Set the minimal clock frequency of a core
+Set the minimal clock frequency of a hardware thread
 @param [in] cpu_id CPU ID
 @param [in] freq Frequency in kHz
 @return Frequency or 0 in case of errors
 */
 extern uint64_t freq_setCpuClockMin(const int cpu_id, const uint64_t freq) __attribute__ ((visibility ("default") ));
-/*! \brief De/Activate turbo mode for core
+/*! \brief De/Activate turbo mode for a hardware thread
 
-De/Activate turbo mode for core
+De/Activate turbo mode for a hardware thread
 @param [in] cpu_id CPU ID
 @param [in] turbo (0=off, 1=on)
 @return 1 or 0 in case of errors
 */
 extern int freq_setTurbo(const int cpu_id, int turbo) __attribute__ ((visibility ("default") ));
-/*! \brief Get state of turbo mode for core
+/*! \brief Get state of turbo mode for a hardware thread
 
-Get state of  turbo mode for core
+Get state of turbo mode for a hardware thread
 @param [in] cpu_id CPU ID
 @return 1=Turbo active or 0=Turbo inactive
 */
 extern int freq_getTurbo(const int cpu_id) __attribute__ ((visibility ("default") ));
-/*! \brief Get the frequency governor of a core
+/*! \brief Get the frequency governor of a hardware thread
 
-Get the frequency governor of a core. The returned string must be freed by the caller.
+Get the frequency governor of a hardware thread. The returned string must be freed by the caller.
 @param [in] cpu_id CPU ID
 @return Governor or NULL in case of errors
 */
 extern char * freq_getGovernor(const int cpu_id ) __attribute__ ((visibility ("default") ));
-/*! \brief Set the frequency governor of a core
+/*! \brief Set the frequency governor of a hardware thread
 
-Set the frequency governor of a core.
+Set the frequency governor of a hardware thread.
 @param [in] cpu_id CPU ID
 @param [in] gov Governor
 @return 1 or 0 in case of errors
 */
 extern int freq_setGovernor(const int cpu_id, const char* gov) __attribute__ ((visibility ("default") ));
-/*! \brief Get the available frequencies of a core
+/*! \brief Get the available frequencies of a hardware thread
 
-Get the available frequencies of a core. The returned string must be freed by the caller.
+Get the available frequencies of a hardware thread. The returned string must be freed by the caller.
 @param [in] cpu_id CPU ID
 @return String with available frequencies or NULL in case of errors
 */
 extern char * freq_getAvailFreq(const int cpu_id ) __attribute__ ((visibility ("default") ));
-/*! \brief Get the available frequency governors of a core
+/*! \brief Get the available frequency governors of a hardware thread
 
-Get the available frequency governors of a core. The returned string must be freed by the caller.
+Get the available frequency governors of a hardware thread. The returned string must be freed by the caller.
 @param [in] cpu_id CPU ID
 @return String with available frequency governors or NULL in case of errors
 */
@@ -1800,6 +1827,7 @@ typedef struct {
     int devid; /*!< \brief Device ID  */
     int numaNode; /*!< \brief Closest NUMA domain to the device */
     char* name; /*!< \brief Name of the device */
+    char* short_name; /*!< \brief Short name of the device */
     uint64_t mem; /*!< \brief Total memory of device */
     int ccapMajor; /*!< \brief Major number of device's compute capability */
     int ccapMinor; /*!< \brief Minor number of device's compute capability */
@@ -1927,15 +1955,16 @@ Reset the values of all configured counters and timers.
 extern int likwid_gpuMarkerResetRegion(const char* regionTag) __attribute__ ((visibility ("default") ));
 /*! \brief Get accumulated data of a code region
 
-Get the accumulated data of the current thread for the given regionTag.
+Get the accumulated data of the GPUs for the given regionTag. If the operation fails, nr_gpus and nr_events are set to zero.
+
 @param regionTag [in] Print data using this string
-@param nr_gpus [in,out] Length of first dimension of the arrys. Afterwards the actual count of GPUs.
-@param nr_events [in,out] Length of events array
+@param nr_gpus [in,out] Length of first dimension of the arrys. Afterwards the actual count of GPUs and consequently the length of events, time and count.
+@param nr_events [in,out] Length of events array. Afterwards the actual count of events in the second dimension of events.
 @param events [out] Events array for the intermediate results
-@param time [out] Accumulated measurement time
-@param count [out] Call count of the code region
+@param time [out] Accumulated measurement times per GPU
+@param count [out] Call counts of the code region per GPU
 */
-extern void likwid_gpuMarkerGetRegion(const char* regionTag, int* nr_gpus, int* nr_events, double** events, double **time, int **count) __attribute__ ((visibility ("default") ));
+extern void likwid_gpuMarkerGetRegion(const char* regionTag, int* nr_gpus, int* nr_events, double** events, double *time, int *count) __attribute__ ((visibility ("default") ));
 
 /*! \brief Read the output file of the NvMarker API
 @param [in] filename Filename with NvMarker API results
@@ -2027,9 +2056,9 @@ double nvmon_getMetricOfRegionGpu(int region, int metricId, int gpuId) __attribu
 It holds the name, the description and the limitation string for one event.
 */
 typedef struct {
-    char* name; /*! \brief Name of the event */
-    char* desc; /*! \brief Description of the event */
-    char* limit; /*! \brief Limitation string of the event, commonly 'GPU' */
+    char* name; /*!< \brief Name of the event */
+    char* desc; /*!< \brief Description of the event */
+    char* limit; /*!< \brief Limitation string of the event, commonly 'GPU' */
 } NvmonEventListEntry;
 
 /*! \brief Output list from nvmon_getEventsOfGpu with all supported events
@@ -2037,8 +2066,8 @@ typedef struct {
 Output list from nvmon_getEventsOfGpu with all supported events
 */
 typedef struct {
-    int numEvents; /*! \brief Number of events */
-    NvmonEventListEntry *events; /*! \brief List of events */
+    int numEvents; /*!< \brief Number of events */
+    NvmonEventListEntry *events; /*!< \brief List of events */
 } NvmonEventList;
 /** \brief Pointer for exporting the NvmonEventList data structure */
 typedef NvmonEventList* NvmonEventList_t;
@@ -2149,7 +2178,7 @@ Get the metric result of all measurement cycles. It reads all raw results for th
 @param [in] gpuId ID of the GPU that should be read
 @return The metric result
 */
-double nvmon_getMetric(int groupId, int metricId, int gpuId);
+double nvmon_getMetric(int groupId, int metricId, int gpuId) __attribute__ ((visibility ("default") ));
 /*! \brief Get the last metric result of the specified group, counter and GPU (Nvmon)
 
 Get the metric result of the last measurement cycle. It reads all raw results for the given groupId and gpuId.
@@ -2158,7 +2187,7 @@ Get the metric result of the last measurement cycle. It reads all raw results fo
 @param [in] gpuId ID of the GPU that should be read
 @return The metric result
 */
-double nvmon_getLastMetric(int groupId, int metricId, int gpuId);
+double nvmon_getLastMetric(int groupId, int metricId, int gpuId) __attribute__ ((visibility ("default") ));
 /*! \brief Get the number of configured event groups (Nvmon)
 
 @return Number of groups
@@ -2250,9 +2279,13 @@ char* nvmon_getGroupInfoLong(int groupId) __attribute__ ((visibility ("default")
 
 Checks the configured performance group path for the current GPU and
 returns all found group names
+@param [in] gpuId Get groups for a specific GPU
+@param [out] groups List of group names
+@param [out] shortinfos List of short information string about group
+@param [out] longinfos List of long information string about group
 @return Amount of found performance groups
 */
-int nvmon_getGroups(char*** groups, char*** shortinfos, char*** longinfos) __attribute__ ((visibility ("default") ));
+int nvmon_getGroups(int gpuId, char*** groups, char*** shortinfos, char*** longinfos) __attribute__ ((visibility ("default") ));
 /*! \brief Free all group information (Nvmon)
 
 @param [in] nrgroups Number of groups
