@@ -5,14 +5,14 @@
  *
  *      Description:  Benchmarking framework for likwid-bench
  *
- *      Version:   5.0
- *      Released:  10.11.2019
+ *      Version:   5.2.2
+ *      Released:  26.07.2022
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *               Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2022 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -101,7 +101,7 @@ runTest(void* arg)
 
     /* pin the thread */
     likwid_pinThread(myData->processors[threadId]);
-    printf("Group: %d Thread %d Global Thread %d running on core %d - Vector length %llu Offset %zd\n",
+    printf("Group: %d Thread %d Global Thread %d running on hwthread %d - Vector length %llu Offset %zd\n",
             data->groupId,
             threadId,
             data->globalThreadId,
@@ -484,7 +484,7 @@ getIterSingle(void* arg)
     int threadId = 0;
     size_t offset = 0;
     size_t size = 0, vecsize = 0;
-    size_t i;
+    size_t i, j;
     ThreadData* data;
     ThreadUserData* myData;
     TimerData time;
@@ -497,7 +497,7 @@ getIterSingle(void* arg)
     threadId = data->threadId;
 
     //size = myData->size - (myData->size % myData->test->stride);
-    vecsize = myData->size;
+    vecsize = myData->size / data->numberOfThreads;
     size = myData->size / data->numberOfThreads;
 
     size -= (size % myData->test->stride);
@@ -509,6 +509,64 @@ getIterSingle(void* arg)
     printf("Automatic iteration count detection:");
 #endif
 
+    switch ( myData->test->type )
+    {
+        case SINGLE:
+            {
+                float* sptr;
+                for (i=0; i <  myData->test->streams; i++)
+                {
+                    sptr = (float*) myData->streams[i];
+                    sptr +=  offset;
+                    if (myData->init_per_thread)
+                    {
+                        for (j = 0; j < vecsize; j++)
+                        {
+                            sptr[j] = 1.0;
+                        }
+                    }
+                    myData->streams[i] = (float*) sptr;
+                }
+            }
+            break;
+        case INT:
+            {
+                int* sptr;
+                for (i=0; i <  myData->test->streams; i++)
+                {
+                    sptr = (int*) myData->streams[i];
+                    sptr +=  offset;
+                    if (myData->init_per_thread)
+                    {
+                        for (j = 0; j < vecsize; j++)
+                        {
+                            sptr[j] = 1;
+                        }
+                    }
+                    myData->streams[i] = (int*) sptr;
+                }
+            }
+            break;
+        case DOUBLE:
+            {
+                double* dptr;
+                for (i=0; i <  myData->test->streams; i++)
+                {
+                    dptr = (double*) myData->streams[i];
+                    dptr +=  offset;
+                    if (myData->init_per_thread)
+                    {
+                        for (j = 0; j < vecsize; j++)
+                        {
+                            dptr[j] = 1.0;
+                        }
+                    }
+                    myData->streams[i] = (double*) dptr;
+                }
+            }
+            break;
+    }
+
     switch ( myData->test->streams ) {
         case STREAM_1:
             MEASURE(func(size,myData->streams[0]));
