@@ -5,13 +5,13 @@
  *
  *      Description:  Interface to the access daemon for the access module.
  *
- *      Version:   5.1.0
- *      Released:  20.11.2020
+ *      Version:   5.2.2
+ *      Released:  26.07.2022
  *
  *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2022 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -53,6 +53,10 @@
 #include <configuration.h>
 #include <affinity.h>
 
+#if defined(__x86_64__) || defined(__i386__)
+#include <access_x86_rdpmc.h>
+#endif
+
 #define gettid() syscall(SYS_gettid)
 
 /* #####   VARIABLES  -  LOCAL TO THIS SOURCE FILE   ###################### */
@@ -68,6 +72,7 @@ static pthread_mutex_t globalLock = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t *cpuLocks = NULL;
 
 /* #####   FUNCTION DEFINITIONS  -  LOCAL TO THIS SOURCE FILE   ########### */
+void __attribute__((destructor (104))) close_access_client(void);
 
 static char*
 access_client_strerror(AccessErrorType det)
@@ -102,6 +107,21 @@ access_client_errno(AccessErrorType det)
     }
 }
 
+static void access_client_signal_catcher(int sig) {
+    close_access_client();
+}
+
+static int
+access_client_catch_signal()
+{
+    struct sigaction sia;
+    sia.sa_handler = access_client_signal_catcher;
+    sigemptyset(&sia.sa_mask);
+    sia.sa_flags = SA_ONSTACK;
+    sigaction(SIGCHLD, &sia, NULL);
+    return 0;
+}
+
 static int
 access_client_startDaemon(int cpu_id)
 {
@@ -229,6 +249,9 @@ access_client_init(int cpu_id)
             pthread_mutex_init(&cpuLocks[i], NULL);
         }
     }
+#if defined(__x86_64__) || defined(__i386__)
+    access_x86_rdpmc_init(cpu_id);
+#endif
     if (masterPid != 0 && gettid() == masterPid)
     {
         return 0;
@@ -312,6 +335,12 @@ access_client_read(PciDeviceIndex dev, const int cpu_id, uint32_t reg, uint64_t
         record.cpu = affinity_thread2socket_lookup[cpu_id];
         record.device = dev;
     }
+    else
+    {
+#if defined(__x86_64__) || defined(__i386__)
+        access_x86_rdpmc_read(cpu_id, reg, data);
+#endif
+    }
     if (socket != -1)
     {
         record.reg = reg;
@@ -467,6 +496,9 @@ access_client_finalize(int cpu_id)
         globalSocket = -1;
     }
     masterPid = 0;
+#if defined(__x86_64__) || defined(__i386__)
+    access_x86_rdpmc_finalize(cpu_id);
+#endif
 }
 
 int
