@@ -7,13 +7,13 @@
  *      Description: A wrapper script to pin threads spawned by MPI processes and
  *                   measure hardware performance counters
  *
- *      Version:   5.2.1
- *      Released:  03.12.2021
+ *      Version:   5.2.2
+ *      Released:  26.07.2022
  *
  *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
+ *      Copyright (C) 2022 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -269,9 +269,7 @@ local function executeOpenMPI(wrapperscript, hostfile, env, nrNodes)
         elseif ver2 == "6" then
             bindstr = "--bind-to-none"
         end
-    elseif ver1 == 2 then
-        bindstr = "--bind-to none"
-    elseif ver1 == 3 then
+    elseif ver1 >= 2 then
         bindstr = "--bind-to none"
     end
 
@@ -401,7 +399,7 @@ local function executeIntelMPI(wrapperscript, hostfile, env, nrNodes)
             print_stdout(string.format("EXEC: %s/mpiexec -perhost %d %s -np %d %s", path, ppn, envstr, np, wrapperscript))
             print_stdout(string.format("EXEC: %s/mpdallexit", path))
         else
-            print_stdout(string.format("%s %s -f %s -np %d -perhost %d %s",mpiexecutable, envstr, hostfile, np, ppn, wrapperscript))
+            print_stdout(string.format("%s %s -machinefile %s -np %d -perhost %d %s",mpiexecutable, envstr, hostfile, np, ppn, wrapperscript))
         end
     end
 
@@ -412,7 +410,7 @@ local function executeIntelMPI(wrapperscript, hostfile, env, nrNodes)
         ret = os.execute(string.format("%s/mpiexec -perhost %d %s -np %d %s", path, ppn, envstr, np, wrapperscript))
         ret = os.execute(string.format("%s/mpdallexit", path))
     else
-        ret = os.execute(string.format("%s %s -f %s -np %d -perhost %d %s",mpiexecutable, envstr, hostfile, np, ppn, wrapperscript))
+        ret = os.execute(string.format("%s %s -machinefile %s -np %d -perhost %d %s",mpiexecutable, envstr, hostfile, np, ppn, wrapperscript))
     end
     return ret
 end
@@ -1435,22 +1433,30 @@ local function writeWrapperScript(scriptname, execStr, hosts, envsettings, outpu
         end
     end
 
+    if np == nil then np = 1 end
+    local i_np = math.tointeger(np)
+    if i_np == nil then i_np = 1 end
+
+    if ppn == nil then ppn = 1 end
+    local i_ppn = math.tointeger(ppn)
+    if i_ppn == nil then i_ppn = 1 end
+
     if mpitype == "openmpi" then
         glsize_var = "$OMPI_COMM_WORLD_SIZE"
         glrank_var = "${OMPI_COMM_WORLD_RANK:-$(($GLOBALSIZE * 2))}"
         losize_var = "$OMPI_COMM_WORLD_LOCAL_SIZE"
     elseif mpitype == "intelmpi" then
         glrank_var = "${PMI_RANK:-$(($GLOBALSIZE * 2))}"
-        glsize_var = tostring(math.tointeger(np))
-        losize_var = tostring(math.tointeger(ppn))
+        glsize_var = tostring(i_np)
+        losize_var = tostring(i_ppn)
     elseif mpitype == "mvapich2" then
         glrank_var = "${PMI_RANK:-$(($GLOBALSIZE * 2))}"
-        glsize_var = tostring(math.tointeger(np))
-        losize_var = tostring(math.tointeger(ppn))
+        glsize_var = tostring(i_np)
+        losize_var = tostring(i_ppn)
     elseif mpitype == "slurm" then
         glrank_var = "${SLURM_PROCID:-$(($GLOBALSIZE * 2))}"
-        glsize_var = tostring(math.tointeger(np))
-        losize_var = string.format("${SLURM_NTASKS_PER_NODE:-%d}", math.tointeger(ppn))
+        glsize_var = tostring(i_np)
+        losize_var = string.format("${SLURM_NTASKS_PER_NODE:-%d}", i_ppn)
     else
         print_stderr("Invalid MPI vendor "..mpitype)
         return
@@ -1523,6 +1529,7 @@ local function writeWrapperScript(scriptname, execStr, hosts, envsettings, outpu
     f:write("GLOBALSIZE="..glsize_var.."\n")
     f:write("GLOBALRANK="..glrank_var.."\n")
     if os.getenv("OMP_NUM_THREADS") == nil then
+        if tpp == nil then tpp = 1 end
         f:write(string.format("export OMP_NUM_THREADS=%d\n", tpp))
     else
         f:write(string.format("export OMP_NUM_THREADS=%s\n", os.getenv("OMP_NUM_THREADS")))
@@ -2465,24 +2472,20 @@ if skipStr == "" then
             skipStr = '-s 0x7'
         end
     elseif mpitype == "openmpi" then
-        if omptype == "intel" and nrNodes > 1 then
+        if omptype == "intel" then
             skipStr = '-s 0x7'
-        elseif omptype == "intel" and nrNodes == 1 then
-            skipStr = '-s 0x1'
-            if tpp > 1 then
-                skipStr = '-s 0x3'
-            end
-        elseif omptype == "gnu" and nrNodes > 1 then
+        elseif omptype == "gnu" then
             skipStr = '-s 0x7'
-        elseif omptype == "gnu" and nrNodes == 1 then
-            skipStr = '-s 0x0'
-            if tpp > 0 then
-                skipStr = '-s 0x1'
-            end
         end
     elseif mpitype == "slurm" then
-        if omptype == "intel" and nrNodes > 1 then
-            if nrNodes == 1 then
+        if omptype == "intel" then
+            if nrNodes == 1 and tpp == 1 then
+                skipStr = '-s 0x1'
+            else
+                skipStr = '-s 0x3'
+            end
+        elseif omptype == "gnu" then
+            if nrNodes == 1 and tpp == 1 then
                 skipStr = '-s 0x1'
             else
                 skipStr = '-s 0x3'
