@@ -6,13 +6,13 @@
  *
  *      Description:  A application to set the CPU frequency of CPU cores and domains.
  *
- *      Version:   5.1.0
- *      Released:  20.11.2020
+ *      Version:   5.2.2
+ *      Released:  26.07.2022
  *
  *      Author:   Thomas Gruber (tr), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2022 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -127,7 +127,6 @@ function get_base_freq()
         out = f:read("*a")
         freq = tonumber(out)
         f:close()
-        return freq
     end
     f = io.open("/sys/devices/system/cpu/cpu0/cpufreq/bios_limit", "r")
     if f ~= nil then
@@ -142,7 +141,7 @@ function get_base_freq()
         freq = tonumber(out)
         f:close()
     end
-    return freq*1E6
+    return freq*1E3
 end
 
 verbosity = 0
@@ -271,18 +270,16 @@ for i, dom in pairs(affinity["domains"]) do
     end
 end
 
-if cpuinfo["clock"] > 0 then
-    base_freq = cpuinfo["clock"] * 1.E-03
-else
-    base_freq = get_base_freq()
-end
-if not base_freq then
-    base_freq = likwid.getCpuClock() * 1.E-03
-end
-
 likwid.initFreq()
 driver = likwid.getFreqDriver(cpulist[1])
 
+base_freq = likwid.getCpuClockBase(cpulist[1])
+if base_freq == 0 then
+    if get_base_freq() ~= nil then
+        base_freq = get_base_freq()
+    end
+end
+
 if verbosity == 3 then
     print_stdout(string.format("DEBUG: Given CPU expression expands to %d HW Threads:", numthreads))
     local str = "DEBUG: " .. tostring(cpulist[1])
@@ -375,7 +372,7 @@ if do_reset then
     local availfreqs = likwid.getAvailFreq(cpulist[1])
     local availgovs = likwid.getAvailGovs(cpulist[1])
     if driver == "intel_pstate" then
-        availfreqs = {likwid.getConfCpuClockMin(cpulist[1]), likwid.getConfCpuClockMax(cpulist[1])}
+        availfreqs = {likwid.getConfCpuClockMin(cpulist[1]), base_freq}
     end
     if not min_freq then
         min_freq = availfreqs[1]
@@ -464,26 +461,57 @@ if min_u_freq and max_u_freq and max_u_freq < min_u_freq then
     os.exit(1)
 end
 
-if max_freq and (likwid.getTurbo(cpulist[1]) == 1 or (set_turbo and turbo == 1)) and tonumber(max_freq) < base_freq then
-    print_stderr("ERROR: Setting maximal CPU frequency below base CPU frequency with activated Turbo mode is not supported.")
-    likwid.finalizeFreq()
-    os.exit(1)
-end
-if (likwid.getTurbo(cpulist[1]) == 0 and not set_turbo) then
-    local do_exit = false
-    if max_freq and tonumber(max_freq) > base_freq then
-        print_stderr("ERROR: Setting maximal CPU frequency above base CPU frequency with deactivated Turbo mode is not supported.")
-        do_exit = true
-    elseif min_freq and tonumber(min_freq) > base_freq then
-        print_stderr("ERROR: Setting minimal CPU frequency above base CPU frequency with deactivated Turbo mode is not supported.")
-        do_exit = true
-    end
-    if do_exit then
-        likwid.finalizeFreq()
-        os.exit(1)
+if max_freq then
+    if tonumber(max_freq) < base_freq then
+        
+        if (set_turbo and turbo == 1) then
+            print_stderr("ERROR: Setting maximal CPU frequency below base CPU frequency with activated Turbo mode is not supported.")
+            likwid.finalizeFreq()
+            os.exit(1)
+        elseif (likwid.getTurbo(cpulist[1]) == 1 and ((not set_turbo) and turbo == 0)) then
+--            print_stdout("currently invalid setting but we will disable turbo")
+        end
+    elseif tonumber(max_freq) > base_freq then
+        if ((set_turbo and turbo == 0)) then
+            print_stderr("ERROR: Setting maximal CPU frequency above base CPU frequency with deactivated Turbo mode is not supported.")
+            likwid.finalizeFreq()
+            os.exit(1)
+        elseif ((likwid.getTurbo(cpulist[1]) == 0) and not (set_turbo and turbo == 1)) then
+            print_stderr("ERROR: Setting maximal CPU frequency above base CPU frequency when Turbo mode is deactivated is not supported.")
+            likwid.finalizeFreq()
+            os.exit(1)
+        elseif ((likwid.getTurbo(cpulist[1]) == 1) or ((not set_turbo) and turbo == 1)) then
+            print_stdout("currently invalid setting but we will enable turbo")
+        end
     end
 end
 
+if min_freq then
+    if tonumber(min_freq) < base_freq then
+        
+        if (set_turbo and turbo == 1) then
+            print_stderr("ERROR: Setting minimal CPU frequency below base CPU frequency with activated Turbo mode is not supported.")
+            likwid.finalizeFreq()
+            os.exit(1)
+        elseif (likwid.getTurbo(cpulist[1]) == 1 and ((not set_turbo) and turbo == 0)) then
+            print_stdout("currently invalid setting but we will disable turbo")
+        end
+    elseif tonumber(min_freq) > base_freq then
+        if ((set_turbo and turbo == 0)) then
+            print_stderr("ERROR: Setting minimal CPU frequency above base CPU frequency with deactivated Turbo mode is not supported.")
+            likwid.finalizeFreq()
+            os.exit(1)
+        elseif ((likwid.getTurbo(cpulist[1]) == 0) and not (set_turbo and turbo == 1))then
+            print_stderr("ERROR: Setting minimal CPU frequency above base CPU frequency when Turbo mode is deactivated is not supported.")
+            likwid.finalizeFreq()
+            os.exit(1)
+        elseif ((likwid.getTurbo(cpulist[1]) == 1) or ((not set_turbo) and turbo == 1)) then
+--            print_stdout("currently invalid setting but we will enable turbo")
+        end
+    end
+end
+
+
 
 local availfreqs = likwid.getAvailFreq(cpulist[1])
 if driver == "intel_pstate" then
@@ -538,7 +566,6 @@ if set_turbo then
     end
 end
 
-
 if max_first and max_freq then
     for i=1,#cpulist do
         local f = likwid.setCpuClockMax(cpulist[i], max_freq)
@@ -557,11 +584,19 @@ elseif min_first and min_freq then
             local f = likwid.setCpuClockMax(cpulist[i], max_freq)
         end
     end
-else
+elseif min_freq and max_freq then
     for i=1,#cpulist do
         local f = likwid.setCpuClockMin(cpulist[i], min_freq)
         local f = likwid.setCpuClockMax(cpulist[i], max_freq)
     end
+elseif min_freq then
+    for i=1,#cpulist do
+        local f = likwid.setCpuClockMin(cpulist[i], min_freq)
+    end
+elseif max_freq then
+    for i=1,#cpulist do
+        local f = likwid.setCpuClockMax(cpulist[i], max_freq)
+    end
 end
 
 if min_u_freq then
