@@ -5,13 +5,13 @@
  *
  *      Description:  Lua LIKWID interface library
  *
- *      Version:   5.2.1
- *      Released:  03.12.2021
+ *      Version:   5.2.2
+ *      Released:  26.07.2022
  *
  *      Author:   Thomas Gruber (tr), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
+ *      Copyright (C) 2022 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -1222,6 +1222,18 @@ end
 
 likwid.gethostname = gethostname
 
+local function get_perf_event_paranoid()
+    local f = io.open("/proc/sys/kernel/perf_event_paranoid")
+    if f then
+        value = f:read("*l")
+        f:close()
+        return tonumber(value)
+    end
+    return 4
+end
+
+likwid.perf_event_paranoid = get_perf_event_paranoid
+
 local function getjid()
     jid = "X"
     for _, v in pairs({"PBS_JOBID", "SLURM_JOB_ID", "SLURM_JOBID", "LOADL_STEP_ID", "LSB_JOBID" }) do
