@@ -9,13 +9,13 @@
  *                  Allows to turn on and off the Hardware prefetcher
  *                  available.
  *
- *      Version:   5.1.0
- *      Released:  20.11.2020
+ *      Version:   5.2.2
+ *      Released:  26.07.2022
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2022 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -165,7 +165,9 @@ cpuFeatures_update(int cpu)
              (cpuid_info.model == SKYLAKEX) ||
              (cpuid_info.model == KABYLAKE1) ||
              (cpuid_info.model == KABYLAKE2) ||
-             (cpuid_info.model == CANNONLAKE))
+             (cpuid_info.model == CANNONLAKE) ||
+             (cpuid_info.model == COMETLAKE1) ||
+             (cpuid_info.model == COMETLAKE2))
     {
         TEST_FLAG_INV(FEAT_TURBO_MODE,38);
     }
@@ -196,10 +198,13 @@ cpuFeatures_update(int cpu)
             (cpuid_info.model == KABYLAKE2) ||
             (cpuid_info.model == ICELAKE1) ||
             (cpuid_info.model == ICELAKE2) ||
+            (cpuid_info.model == ROCKETLAKE) ||
             (cpuid_info.model == ICELAKEX1) ||
             (cpuid_info.model == ICELAKEX2) ||
             (cpuid_info.model == ATOM_SILVERMONT_GOLD) ||
-            (cpuid_info.model == CANNONLAKE))
+            (cpuid_info.model == CANNONLAKE) ||
+            (cpuid_info.model == COMETLAKE1) ||
+            (cpuid_info.model == COMETLAKE2))
     {
         ret = HPMread(cpu, MSR_DEV, MSR_PREFETCH_ENABLE, &flags);
         if (ret != 0)
@@ -374,8 +379,11 @@ cpuFeatures_enable(int cpu, CpuFeature type, int print)
             (cpuid_info.model == KABYLAKE1) ||
             (cpuid_info.model == KABYLAKE2) ||
             (cpuid_info.model == CANNONLAKE) ||
+            (cpuid_info.model == COMETLAKE1) ||
+            (cpuid_info.model == COMETLAKE2) ||
             (cpuid_info.model == ICELAKE1) ||
             (cpuid_info.model == ICELAKE2) ||
+            (cpuid_info.model == ROCKETLAKE) ||
             (cpuid_info.model == ICELAKEX1) ||
             (cpuid_info.model == ICELAKEX2) ||
             (cpuid_info.model == ATOM_SILVERMONT_GOLD))
@@ -535,8 +543,11 @@ cpuFeatures_disable(int cpu, CpuFeature type, int print)
             (cpuid_info.model == KABYLAKE1) ||
             (cpuid_info.model == KABYLAKE2) ||
             (cpuid_info.model == CANNONLAKE) ||
+            (cpuid_info.model == COMETLAKE1) ||
+            (cpuid_info.model == COMETLAKE2) ||
             (cpuid_info.model == ICELAKE1) ||
             (cpuid_info.model == ICELAKE2) ||
+            (cpuid_info.model == ROCKETLAKE) ||
             (cpuid_info.model == ICELAKEX1) ||
             (cpuid_info.model == ICELAKEX2) ||
             (cpuid_info.model == ATOM_SILVERMONT_GOLD))
