@@ -6,14 +6,14 @@
  *      Description:  Module implementing an interface for frequency manipulation
  *                    Module for manipuating CPU frequencies
  *
- *      Version:   5.1.0
- *      Released:  20.11.2020
+ *      Version:   5.2.2
+ *      Released:  26.07.2022
  *
  *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *                Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2022 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -866,6 +866,56 @@ static int getIntelHWP(const int cpu_id)
 #endif
 }
 
+static int getBaseFreq(const int cpu_id)
+{
+    int err = 0;
+
+    if (!lock_check())
+    {
+        fprintf(stderr,"Access to frequency backend is locked.\n");
+        return 0;
+    }
+#ifdef LIKWID_USE_PERFEVENT
+    fprintf(stderr,"Cannot read base frequency with ACCESSMODE=perf_event.\n");
+    return -1;
+#else
+    if (!HPMinitialized())
+    {
+        HPMinit();
+        own_hpm = 1;
+        err = HPMaddThread(cpu_id);
+        if (err != 0)
+        {
+            ERROR_PLAIN_PRINT(Cannot get access to MSRs)
+            return err;
+        }
+    }
+    else
+    {
+        err = HPMaddThread(cpu_id);
+        if (err != 0)
+        {
+            ERROR_PLAIN_PRINT(Cannot get access to MSRs)
+            return err;
+        }
+    }
+
+    uint64_t tmp = 0x0ULL;
+    err = HPMread(cpu_id, MSR_DEV, MSR_PLATFORM_INFO, &tmp);
+    if (err)
+    {
+        timer_init();
+        uint64_t tmp = timer_getCpuClock()/1000;
+        return tmp;
+    }
+    else
+    {
+        tmp = extractBitField(tmp,8,8);
+        return 100000 * tmp;
+    }
+
+#endif
+}
 
 int
 _freqInit(void)
@@ -1031,6 +1081,15 @@ int freq_setGovernor(const int cpu_id, const char* gov)
     return -EINVAL;
 }
 
+uint64_t freq_getCpuClockBase(const int cpu_id)
+{
+    uint64_t f = getBaseFreq(cpu_id);
+    if (f > 0)
+    {
+        return f;
+    }
+    return 0;
+}
 
 uint64_t freq_getCpuClockCurrent(const int cpu_id)
 {
@@ -1212,8 +1271,3 @@ int freq_setTurbo(const int cpu_id, const int turbo)
         return setIntelTurbo(cpu_id, turbo);
     return 1;
 }
-
-void __attribute__((destructor (104))) close_frequency_cpu(void)
-{
-    _freqFinalize();
-}
