@@ -5,13 +5,13 @@
  *
  *      Description:  Header File of perfmon module for perf_event kernel interface.
  *
- *      Version:   5.1.0
- *      Released:  20.11.2020
+ *      Version:   5.2.2
+ *      Released:  26.07.2022
  *
  *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2022 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -43,10 +43,12 @@
 extern char** translate_types;
 static int** cpu_event_fds = NULL;
 static int active_cpus = 0;
-static int paranoid_level = -1;
-static int informed_paranoid = 0;
+static int perf_event_initialized = 0;
+/*static int informed_paranoid = 0;*/
 static int running_group = -1;
 static int perf_event_num_cpus = 0;
+static int perf_disable_uncore = 0;
+static int perf_event_paranoid = -1;
 
 static long
 perf_event_open(struct perf_event_attr *hw_event, pid_t pid,
@@ -67,8 +69,8 @@ int perfevent_paranoid_value()
     fd = fopen("/proc/sys/kernel/perf_event_paranoid", "r");
     if (fd == NULL)
     {
-        fprintf(stderr, "ERROR: Linux kernel has no perf_event support\n");
-        fprintf(stderr, "ERROR: Cannot open file /proc/sys/kernel/perf_event_paranoid\n");
+        errno = EPERM;
+        ERROR_PRINT(Linux kernel has no perf_event support. Cannot access /proc/sys/kernel/perf_event_paranoid);
         return paranoid;
     }
     size_t read = fread(buff, sizeof(char), 100, fd);
@@ -82,29 +84,19 @@ int perfevent_paranoid_value()
 
 int perfmon_init_perfevent(int cpu_id)
 {
-    int paranoid = -1;
-    if (!informed_paranoid)
+    perf_event_paranoid = perfevent_paranoid_value();
+    if (getuid() != 0 && perf_event_paranoid > 2)
     {
-        paranoid_level = perfevent_paranoid_value();
-#if defined(__x86_64__) || defined(__i386__) || defined(_ARCH_PPC)
-        if (paranoid_level > 0 && getuid() != 0)
-        {
-            fprintf(stderr, "WARN: Linux kernel configured with paranoid level %d\n", paranoid_level);
-            fprintf(stderr, "WARN: Paranoid level 0 or root access is required to measure Uncore counters\n");
-        }
-#endif
-#if defined(__ARM_ARCH_8A) || defined(__ARM_ARCH_7A__)
-        if (paranoid_level > 1 && getuid() != 0)
-        {
-            fprintf(stderr, "WARN: Linux kernel configured with paranoid level %d\n", paranoid_level);
-        }
-#endif
-        informed_paranoid = 1;
+        errno = EPERM;
+        ERROR_PRINT(Cannot use performance monitoring with perf_event_paranoid = %d, perf_event_paranoid);
+        return -(cpu_id+1);
     }
+
     lock_acquire((int*) &tile_lock[affinity_thread2core_lookup[cpu_id]], cpu_id);
     lock_acquire((int*) &socket_lock[affinity_thread2socket_lookup[cpu_id]], cpu_id);
     lock_acquire((int*) &numa_lock[affinity_thread2numa_lookup[cpu_id]], cpu_id);
     lock_acquire((int*) &sharedl3_lock[affinity_thread2sharedl3_lookup[cpu_id]], cpu_id);
+    lock_acquire((int*) &die_lock[affinity_thread2die_lookup[cpu_id]], cpu_id);
     if (cpu_event_fds == NULL)
     {
         cpu_event_fds = malloc(cpuid_topology.numHWThreads * sizeof(int*));
@@ -122,6 +114,7 @@ int perfmon_init_perfevent(int cpu_id)
         active_cpus += 1;
     }
     perf_event_num_cpus = cpuid_topology.numHWThreads;
+    perf_event_initialized = 1;
     return 0;
 }
 
@@ -144,6 +137,8 @@ static char* perfEventOptionNames[] = {
     [EVENT_OPTION_MATCH0] = "offcore_rsp",
     [EVENT_OPTION_MATCH1] = "offcore_rsp",
     [EVENT_OPTION_TID] = "tid_en",
+    [EVENT_OPTION_CID] = "cid",
+    [EVENT_OPTION_SLICE] = "slice",
     [EVENT_OPTION_STATE] = "filter_state",
     [EVENT_OPTION_NID] = "filter_nid",
     [EVENT_OPTION_OPCODE] = "filter_opc",
@@ -160,6 +155,7 @@ static char* perfEventOptionNames[] = {
 #ifdef _ARCH_PPC
     [EVENT_OPTION_PMC] = "pmc",
     [EVENT_OPTION_PMCXSEL] = "pmcxsel",
+    [EVENT_OPTION_UNCORE_CONFIG] = "event",
 #endif
 };
 
@@ -254,25 +250,47 @@ int perf_fixed_setup(struct perf_event_attr *attr, RegisterIndex index, PerfmonE
     {
         attr->exclude_kernel = 1;
         attr->exclude_hv = 1;
-        if (strcmp(event->name, "INSTR_RETIRED_ANY") == 0)
+        if (strncmp(event->name, "INSTR_RETIRED_ANY", 18) == 0)
         {
             attr->config = PERF_COUNT_HW_INSTRUCTIONS;
             ret = 0;
         }
-        if (strcmp(event->name, "CPU_CLK_UNHALTED_CORE") == 0 ||
-            strcmp(event->name, "ACTUAL_CPU_CLOCK") == 0 ||
-            strcmp(event->name, "APERF") == 0)
+        if (strncmp(event->name, "CPU_CLK_UNHALTED_CORE", 22) == 0 ||
+            strncmp(event->name, "ACTUAL_CPU_CLOCK", 17) == 0 ||
+            strncmp(event->name, "APERF", 5) == 0)
         {
             attr->config = PERF_COUNT_HW_CPU_CYCLES;
             ret = 0;
         }
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3,3,0)
-        if (strcmp(event->name, "CPU_CLK_UNHALTED_REF") == 0)
+        if (strncmp(event->name, "CPU_CLK_UNHALTED_REF", 21) == 0)
         {
             attr->config = PERF_COUNT_HW_REF_CPU_CYCLES;
             ret = 0;
         }
 #endif
+        if (cpuid_info.isIntel)
+        {
+            switch(cpuid_info.model)
+            {
+                case ICELAKE1:
+                case ICELAKE2:
+                case ICELAKEX1:
+                case ICELAKEX2:
+                case ROCKETLAKE:
+                case COMETLAKE1:
+                case COMETLAKE2:
+                case TIGERLAKE1:
+                case TIGERLAKE2:
+                case SNOWRIDGEX:
+                    if (strncmp(event->name, "TOPDOWN_SLOTS", 13) == 0)
+                    {
+                        attr->config = 0x0400;
+                        attr->type = PERF_TYPE_RAW;
+                        ret = 0;
+                    }
+            }
+        }
     }
     else
     {
@@ -320,6 +338,17 @@ int perf_fixed_setup(struct perf_event_attr *attr, RegisterIndex index, PerfmonE
     return ret;
 }
 
+int perf_perf_setup(struct perf_event_attr *attr, RegisterIndex index, PerfmonEvent *event)
+{
+    attr->type = PERF_TYPE_HARDWARE;
+    attr->exclude_kernel = 1;
+    attr->exclude_hv = 1;
+    attr->disabled = 1;
+    attr->inherit = 1;
+    attr->config = event->eventId;
+    return 0;
+}
+
 int perf_pmc_setup(struct perf_event_attr *attr, RegisterIndex index, PerfmonEvent *event)
 {
     uint64_t offcore_flags = 0x0ULL;
@@ -473,9 +502,10 @@ int perf_uncore_setup(struct perf_event_attr *attr, RegisterType type, PerfmonEv
     int perf_type = 0;
     PERF_EVENT_PMC_OPT_REGS reg = PERF_EVENT_INVAL_REG;
     int start = 0, end = -1;
-    if (paranoid_level > 0 && getuid() != 0)
+    uint64_t eventConfig = 0x0;
+    if (perf_event_paranoid > 0 && getuid() != 0)
     {
-        return 1;
+        return EPERM;
     }
     attr->type = 0;
     ret = sprintf(checkfolder, "%s", translate_types[type]);
@@ -507,34 +537,59 @@ int perf_uncore_setup(struct perf_event_attr *attr, RegisterType type, PerfmonEv
     attr->type = perf_type;
     attr->disabled = 1;
     attr->inherit = 1;
-    //attr->config = (event->umask<<8) + event->eventId;
+#ifdef _ARCH_PPC
+    eventConfig = (event->umask<<8)|event->eventId;
+#else
+    eventConfig = event->eventId;
+#endif
     getEventOptionConfig(translate_types[type], EVENT_OPTION_GENERIC_CONFIG, &reg, &start, &end);
     switch(reg)
     {
         case PERF_EVENT_CONFIG_REG:
-            attr->config |= create_mask(event->eventId, start, end);
+            attr->config |= create_mask(eventConfig, start, end);
             break;
         case PERF_EVENT_CONFIG1_REG:
-            attr->config1 |= create_mask(event->eventId, start, end);
+            attr->config1 |= create_mask(eventConfig, start, end);
             break;
         case PERF_EVENT_CONFIG2_REG:
-            attr->config2 |= create_mask(event->eventId, start, end);
+            attr->config2 |= create_mask(eventConfig, start, end);
             break;
     }
-    getEventOptionConfig(translate_types[type], EVENT_OPTION_GENERIC_UMASK, &reg, &start, &end);
-    switch(reg)
+#ifdef _ARCH_PPC
+    if (reg == PERF_EVENT_INVAL_REG)
     {
-        case PERF_EVENT_CONFIG_REG:
-            attr->config |= create_mask(event->umask, start, end);
-            break;
-        case PERF_EVENT_CONFIG1_REG:
-            attr->config1 |= create_mask(event->umask, start, end);
-            break;
-        case PERF_EVENT_CONFIG2_REG:
-            attr->config2 |= create_mask(event->umask, start, end);
-            break;
+        getEventOptionConfig(translate_types[type], EVENT_OPTION_UNCORE_CONFIG, &reg, &start, &end);
+        switch(reg)
+        {
+            case PERF_EVENT_CONFIG_REG:
+                attr->config |= create_mask(eventConfig, start, end);
+                break;
+            case PERF_EVENT_CONFIG1_REG:
+                attr->config1 |= create_mask(eventConfig, start, end);
+                break;
+            case PERF_EVENT_CONFIG2_REG:
+                attr->config2 |= create_mask(eventConfig, start, end);
+                break;
+        }
     }
-
+#else
+    if (event->umask != 0x0)
+    {
+        getEventOptionConfig(translate_types[type], EVENT_OPTION_GENERIC_UMASK, &reg, &start, &end);
+        switch(reg)
+        {
+            case PERF_EVENT_CONFIG_REG:
+                attr->config |= create_mask(event->umask, start, end);
+                break;
+            case PERF_EVENT_CONFIG1_REG:
+                attr->config1 |= create_mask(event->umask, start, end);
+                break;
+            case PERF_EVENT_CONFIG2_REG:
+                attr->config2 |= create_mask(event->umask, start, end);
+                break;
+        }
+    }
+#endif
 
     //attr->exclusive = 1;
     if (event->numberOfOptions > 0)
@@ -579,8 +634,6 @@ int perf_uncore_setup(struct perf_event_attr *attr, RegisterType type, PerfmonEv
 }
 
 
-
-
 int perfmon_setupCountersThread_perfevent(
         int thread_id,
         PerfmonEventSet* eventSet)
@@ -592,17 +645,26 @@ int perfmon_setupCountersThread_perfevent(
     int is_uncore = 0;
     pid_t allpid = -1;
     unsigned long allflags = 0;
+    char* env_perf_pid = getenv("LIKWID_PERF_PID");
 
-    if (getenv("LIKWID_PERF_PID") != NULL)
+    if (!perf_event_initialized)
     {
-        allpid = (pid_t)atoi(getenv("LIKWID_PERF_PID"));
+        return -(thread_id+1);
     }
-    else if (paranoid_level > 0 && getuid() != 0)
+    if (env_perf_pid != NULL)
     {
-        fprintf(stderr, "Cannot setup events without PID of executed application because perf_event_paranoid > 0\n");
-        fprintf(stderr, "You can use either --execpid to track the started application or --perfpid <pid> to monitor another application\n");
-        return -((int)thread_id+1);
+        allpid = (pid_t)atoi(env_perf_pid);
     }
+    if (perf_event_paranoid > 0 && getuid() != 0)
+    {
+        if (allpid == -1)
+        {
+            DEBUG_PRINT(DEBUGLEV_INFO, PID of application required. Use LIKWID_PERF_PID env variable or likwid-perfctr options);
+            return -EPERM;
+        }
+        DEBUG_PRINT(DEBUGLEV_DEVELOP, Using PID %d for perf_event measurements, allpid);
+    }
+
     if (getenv("LIKWID_PERF_FLAGS") != NULL)
     {
         allflags = strtoul(getenv("LIKWID_PERF_FLAGS"), NULL, 16);
@@ -647,6 +709,14 @@ int perfmon_setupCountersThread_perfevent(
                 }
                 VERBOSEPRINTREG(cpu_id, index, attr.config, SETUP_FIXED);
                 break;
+            case PERF:
+                ret = perf_perf_setup(&attr, index, event);
+                if (ret < 0)
+                {
+                    continue;
+                }
+                VERBOSEPRINTREG(cpu_id, index, attr.config, SETUP_PERF);
+                break;
             case PMC:
                 pmc_lock = 1;
 #if defined(__ARM_ARCH_8A)
@@ -761,6 +831,16 @@ int perfmon_setupCountersThread_perfevent(
                         has_lock = 1;
                     }
                 }
+                else if (cpuid_info.family == P6_FAMILY &&
+                         cpuid_info.model == SKYLAKEX &&
+                         cpuid_info.stepping >= 5 &&
+                         cpuid_topology.numDies > cpuid_topology.numSockets)
+                {
+                    if (die_lock[affinity_thread2die_lookup[cpu_id]] == cpu_id)
+                    {
+                        has_lock = 1;
+                    }
+                }
                 else
                 {
                     if (socket_lock[affinity_thread2socket_lookup[cpu_id]] == cpu_id)
@@ -781,33 +861,49 @@ int perfmon_setupCountersThread_perfevent(
         }
         if (ret == 0)
         {
-            if (!is_uncore || has_lock)
+            pid_t curpid = allpid;
+            if (is_uncore && curpid >= 0)
+            {
+                curpid = -1;
+            }
+
+            if (!is_uncore)
             {
-                pid_t curpid = allpid;
-                if (is_uncore && curpid >= 0)
-                    curpid = -1;
                 DEBUG_PRINT(DEBUGLEV_DEVELOP, perf_event_open: cpu_id=%d pid=%d flags=%d, cpu_id, curpid, allflags);
                 cpu_event_fds[cpu_id][index] = perf_event_open(&attr, curpid, cpu_id, -1, allflags);
-                if (cpu_event_fds[cpu_id][index] < 0)
+            }
+            else if ((perf_disable_uncore == 0) && (has_lock))
+            {
+                if (perf_event_paranoid > 0 && getuid() != 0)
                 {
-                    fprintf(stderr, "Setup of event %s on CPU %d failed: %s\n", event->name, cpu_id, strerror(errno));
-                    DEBUG_PRINT(DEBUGLEV_DEVELOP, perf_event_open: cpu_id=%d pid=%d flags=%d, cpu_id, curpid, allflags);
-                    DEBUG_PRINT(DEBUGLEV_DEVELOP, type %d, attr.type);
-                    DEBUG_PRINT(DEBUGLEV_DEVELOP, size %d, attr.size);
-                    DEBUG_PRINT(DEBUGLEV_DEVELOP, config 0x%llX, attr.config);
-                    DEBUG_PRINT(DEBUGLEV_DEVELOP, read_format %d, attr.read_format);
-                    DEBUG_PRINT(DEBUGLEV_DEVELOP, disabled %d, attr.disabled);
-                    DEBUG_PRINT(DEBUGLEV_DEVELOP, inherit %d, attr.inherit);
-                    DEBUG_PRINT(DEBUGLEV_DEVELOP, pinned %d, attr.pinned);
-                    DEBUG_PRINT(DEBUGLEV_DEVELOP, exclusive %d, attr.exclusive);
-                    continue;
+                    DEBUG_PRINT(DEBUGLEV_INFO, Cannot measure Uncore with perf_event_paranoid value = %d, perf_event_paranoid);
+                    perf_disable_uncore = 1;
                 }
-                if (group_fd < 0)
-                {
-                    group_fd = cpu_event_fds[cpu_id][index];
-                    running_group = group_fd;
-                }
-                eventSet->events[i].threadCounter[thread_id].init = TRUE;
+                DEBUG_PRINT(DEBUGLEV_DEVELOP, perf_event_open: cpu_id=%d pid=%d flags=%d, cpu_id, curpid, allflags);
+                cpu_event_fds[cpu_id][index] = perf_event_open(&attr, curpid, cpu_id, -1, allflags);
+            }
+            else
+            {
+                DEBUG_PRINT(DEBUGLEV_INFO, Unknown perf_event_paranoid value = %d, perf_event_paranoid);
+            }
+            if (cpu_event_fds[cpu_id][index] < 0)
+            {
+                ERROR_PRINT(Setup of event %s on CPU %d failed: %s, event->name, cpu_id, strerror(errno));
+                DEBUG_PRINT(DEBUGLEV_DEVELOP, open error: cpu_id=%d pid=%d flags=%d type=%d config=0x%llX disabled=%d inherit=%d exclusive=%d config2=0x%llX, cpu_id, curpid, allflags, attr.type, attr.config, attr.disabled, attr.inherit, attr.exclusive);
+            }
+            if (group_fd < 0)
+            {
+                group_fd = cpu_event_fds[cpu_id][index];
+                running_group = group_fd;
+            }
+            eventSet->events[i].threadCounter[thread_id].init = TRUE;
+        }
+        else if (ret == EPERM)
+        {
+            if (is_uncore && perf_disable_uncore == 0 && perf_event_paranoid > 0 && getuid() != 0)
+            {
+                DEBUG_PRINT(DEBUGLEV_INFO, Cannot measure Uncore with perf_event_paranoid value = %d, perf_event_paranoid);
+                perf_disable_uncore = 1;
             }
         }
     }
@@ -818,6 +914,10 @@ int perfmon_startCountersThread_perfevent(int thread_id, PerfmonEventSet* eventS
 {
     int ret = 0;
     int cpu_id = groupSet->threads[thread_id].processorId;
+    if (!perf_event_initialized)
+    {
+        return -(thread_id+1);
+    }
     for (int i=0;i < eventSet->numberOfEvents;i++)
     {
         if (eventSet->events[i].threadCounter[thread_id].init == TRUE)
@@ -827,7 +927,9 @@ int perfmon_startCountersThread_perfevent(int thread_id, PerfmonEventSet* eventS
                 continue;
             VERBOSEPRINTREG(cpu_id, 0x0, 0x0, RESET_COUNTER);
             ioctl(cpu_event_fds[cpu_id][index], PERF_EVENT_IOC_RESET, 0);
-            eventSet->events[i].threadCounter[thread_id].startData = 0x0ULL;
+            PerfmonCounter *c = &eventSet->events[i].threadCounter[thread_id];
+            c->startData = 0x0ULL;
+            c->counterData = 0x0ULL;
             if (eventSet->events[i].type == POWER)
             {
                 ret = read(cpu_event_fds[cpu_id][index],
@@ -835,7 +937,7 @@ int perfmon_startCountersThread_perfevent(int thread_id, PerfmonEventSet* eventS
                         sizeof(long long));
             }
             VERBOSEPRINTREG(cpu_id, 0x0,
-                            eventSet->events[i].threadCounter[thread_id].startData,
+                            c->startData,
                             START_COUNTER);
             ioctl(cpu_event_fds[cpu_id][index], PERF_EVENT_IOC_ENABLE, 0);
         }
@@ -848,6 +950,10 @@ int perfmon_stopCountersThread_perfevent(int thread_id, PerfmonEventSet* eventSe
     int ret;
     int cpu_id = groupSet->threads[thread_id].processorId;
     long long tmp = 0;
+    if (!perf_event_initialized)
+    {
+        return -(thread_id+1);
+    }
     for (int i=0;i < eventSet->numberOfEvents;i++)
     {
         if (eventSet->events[i].threadCounter[thread_id].init == TRUE)
@@ -862,7 +968,26 @@ int perfmon_stopCountersThread_perfevent(int thread_id, PerfmonEventSet* eventSe
             VERBOSEPRINTREG(cpu_id, cpu_event_fds[cpu_id][index], tmp, READ_COUNTER);
             if (ret == sizeof(long long))
             {
+#if defined(__ARM_ARCH_8A)
+                if (cpuid_info.vendor == FUJITSU_ARM && cpuid_info.part == FUJITSU_A64FX)
+                {
+                    switch (eventSet->events[i].event.eventId) {
+                        case 0x3E8:
+                            tmp *= 256;
+                            break;
+                        case 0x3E0:
+                            if (cpuid_topology.numCoresPerSocket == 24)
+                                tmp *= 36;
+                            else
+                                tmp *= 32;
+                            break;
+                        default:
+                            break;
+                    }
+                }
+#endif
                 eventSet->events[i].threadCounter[thread_id].counterData = tmp;
+
             }
             ioctl(cpu_event_fds[cpu_id][index], PERF_EVENT_IOC_RESET, 0);
             VERBOSEPRINTREG(cpu_id, cpu_event_fds[cpu_id][index], 0x0, RESET_COUNTER);
@@ -876,6 +1001,10 @@ int perfmon_readCountersThread_perfevent(int thread_id, PerfmonEventSet* eventSe
     int ret;
     int cpu_id = groupSet->threads[thread_id].processorId;
     long long tmp = 0;
+    if (!perf_event_initialized)
+    {
+        return -(thread_id+1);
+    }
     for (int i=0;i < eventSet->numberOfEvents;i++)
     {
         if (eventSet->events[i].threadCounter[thread_id].init == TRUE)
@@ -890,6 +1019,24 @@ int perfmon_readCountersThread_perfevent(int thread_id, PerfmonEventSet* eventSe
             VERBOSEPRINTREG(cpu_id, cpu_event_fds[cpu_id][index], tmp, READ_COUNTER);
             if (ret == sizeof(long long))
             {
+#if defined(__ARM_ARCH_8A)
+                if (cpuid_info.vendor == FUJITSU_ARM && cpuid_info.part == FUJITSU_A64FX)
+                {
+                    switch (eventSet->events[i].event.eventId) {
+                        case 0x3E8:
+                            tmp *= 256;
+                            break;
+                        case 0x3E0:
+                            if (cpuid_topology.numCoresPerSocket == 24)
+                                tmp *= 36;
+                            else
+                                tmp *= 32;
+                            break;
+                        default:
+                            break;
+                    }
+                }
+#endif
                 eventSet->events[i].threadCounter[thread_id].counterData = tmp;
             }
             VERBOSEPRINTREG(cpu_id, cpu_event_fds[cpu_id][index], 0x0, UNFREEZE_COUNTER);
