@@ -5,14 +5,14 @@
  *
  *      Description:  Counter Header File of perfmon module for Intel Skylake X.
  *
- *      Version:   5.1.0
- *      Released:  20.11.2020
+ *      Version:   5.2.2
+ *      Released:  26.07.2022
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2022 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -525,34 +525,34 @@ static char* skylakeX_translate_types[NUM_UNITS] = {
     [MBOX5] = "/sys/bus/event_source/devices/uncore_imc_5",
     [MBOX6] = "/sys/bus/event_source/devices/uncore_imc_6",
     [MBOX7] = "/sys/bus/event_source/devices/uncore_imc_7",
-    [CBOX0] = "/sys/bus/event_source/devices/uncore_cbox_0",
-    [CBOX1] = "/sys/bus/event_source/devices/uncore_cbox_1",
-    [CBOX2] = "/sys/bus/event_source/devices/uncore_cbox_2",
-    [CBOX3] = "/sys/bus/event_source/devices/uncore_cbox_3",
-    [CBOX4] = "/sys/bus/event_source/devices/uncore_cbox_4",
-    [CBOX5] = "/sys/bus/event_source/devices/uncore_cbox_5",
-    [CBOX6] = "/sys/bus/event_source/devices/uncore_cbox_6",
-    [CBOX7] = "/sys/bus/event_source/devices/uncore_cbox_7",
-    [CBOX8] = "/sys/bus/event_source/devices/uncore_cbox_8",
-    [CBOX9] = "/sys/bus/event_source/devices/uncore_cbox_9",
-    [CBOX10] = "/sys/bus/event_source/devices/uncore_cbox_10",
-    [CBOX11] = "/sys/bus/event_source/devices/uncore_cbox_11",
-    [CBOX12] = "/sys/bus/event_source/devices/uncore_cbox_12",
-    [CBOX13] = "/sys/bus/event_source/devices/uncore_cbox_13",
-    [CBOX14] = "/sys/bus/event_source/devices/uncore_cbox_14",
-    [CBOX15] = "/sys/bus/event_source/devices/uncore_cbox_15",
-    [CBOX16] = "/sys/bus/event_source/devices/uncore_cbox_16",
-    [CBOX17] = "/sys/bus/event_source/devices/uncore_cbox_17",
-    [CBOX18] = "/sys/bus/event_source/devices/uncore_cbox_18",
-    [CBOX19] = "/sys/bus/event_source/devices/uncore_cbox_19",
-    [CBOX20] = "/sys/bus/event_source/devices/uncore_cbox_20",
-    [CBOX21] = "/sys/bus/event_source/devices/uncore_cbox_21",
-    [CBOX22] = "/sys/bus/event_source/devices/uncore_cbox_22",
-    [CBOX23] = "/sys/bus/event_source/devices/uncore_cbox_23",
-    [CBOX24] = "/sys/bus/event_source/devices/uncore_cbox_24",
-    [CBOX25] = "/sys/bus/event_source/devices/uncore_cbox_25",
-    [CBOX26] = "/sys/bus/event_source/devices/uncore_cbox_26",
-    [CBOX27] = "/sys/bus/event_source/devices/uncore_cbox_27",
+    [CBOX0] = "/sys/bus/event_source/devices/uncore_cha_0",
+    [CBOX1] = "/sys/bus/event_source/devices/uncore_cha_1",
+    [CBOX2] = "/sys/bus/event_source/devices/uncore_cha_2",
+    [CBOX3] = "/sys/bus/event_source/devices/uncore_cha_3",
+    [CBOX4] = "/sys/bus/event_source/devices/uncore_cha_4",
+    [CBOX5] = "/sys/bus/event_source/devices/uncore_cha_5",
+    [CBOX6] = "/sys/bus/event_source/devices/uncore_cha_6",
+    [CBOX7] = "/sys/bus/event_source/devices/uncore_cha_7",
+    [CBOX8] = "/sys/bus/event_source/devices/uncore_cha_8",
+    [CBOX9] = "/sys/bus/event_source/devices/uncore_cha_9",
+    [CBOX10] = "/sys/bus/event_source/devices/uncore_cha_10",
+    [CBOX11] = "/sys/bus/event_source/devices/uncore_cha_11",
+    [CBOX12] = "/sys/bus/event_source/devices/uncore_cha_12",
+    [CBOX13] = "/sys/bus/event_source/devices/uncore_cha_13",
+    [CBOX14] = "/sys/bus/event_source/devices/uncore_cha_14",
+    [CBOX15] = "/sys/bus/event_source/devices/uncore_cha_15",
+    [CBOX16] = "/sys/bus/event_source/devices/uncore_cha_16",
+    [CBOX17] = "/sys/bus/event_source/devices/uncore_cha_17",
+    [CBOX18] = "/sys/bus/event_source/devices/uncore_cha_18",
+    [CBOX19] = "/sys/bus/event_source/devices/uncore_cha_19",
+    [CBOX20] = "/sys/bus/event_source/devices/uncore_cha_20",
+    [CBOX21] = "/sys/bus/event_source/devices/uncore_cha_21",
+    [CBOX22] = "/sys/bus/event_source/devices/uncore_cha_22",
+    [CBOX23] = "/sys/bus/event_source/devices/uncore_cha_23",
+    [CBOX24] = "/sys/bus/event_source/devices/uncore_cha_24",
+    [CBOX25] = "/sys/bus/event_source/devices/uncore_cha_25",
+    [CBOX26] = "/sys/bus/event_source/devices/uncore_cha_26",
+    [CBOX27] = "/sys/bus/event_source/devices/uncore_cha_27",
     [BBOX0] = "/sys/bus/event_source/devices/uncore_m2m_0",
     [BBOX1] = "/sys/bus/event_source/devices/uncore_m2m_1",
     [WBOX] = "/sys/bus/event_source/devices/uncore_pcu",
