@@ -5,14 +5,14 @@
  *
  *      Description:  Counter Header File of perfmon module for Intel Skylake.
  *
- *      Version:   5.1.0
- *      Released:  20.11.2020
+ *      Version:   5.2.2
+ *      Released:  26.07.2022
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2022 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -29,8 +29,8 @@
  * =======================================================================================
  */
 
-#define NUM_COUNTERS_SKYLAKE 34
-#define NUM_COUNTERS_CORE_SKYLAKE 13
+#define NUM_COUNTERS_SKYLAKE 37
+#define NUM_COUNTERS_CORE_SKYLAKE 16
 #define NUM_COUNTERS_UNCORE_SKYLAKE 21
 
 #define SKL_VALID_OPTIONS_FIXED EVENT_OPTION_ANYTHREAD_MASK|EVENT_OPTION_COUNT_KERNEL_MASK
@@ -81,14 +81,19 @@ static RegisterMap skylake_counter_map[NUM_COUNTERS_SKYLAKE] = {
     {"MBOX0C2", PMC31, MBOX0, 0x0, 0x2, 0, PCI_IMC_DEVICE_0_CH_0},
     {"MBOX0TMP0", PMC32, MBOX0TMP, 0x0, 0x3, 0, PCI_IMC_DEVICE_0_CH_0},
     {"MBOX0TMP1", PMC33, MBOX0TMP, 0x0, 0x4, 0, PCI_IMC_DEVICE_0_CH_0},
+    /* PERF */
+    {"MPERF", PMC34, PERF, 0, MSR_MPERF, 0, 0, EVENT_OPTION_NONE_MASK},
+    {"APERF", PMC35, PERF, 0, MSR_APERF, 0, 0, EVENT_OPTION_NONE_MASK},
+    {"PPERF", PMC36, PERF, 0, MSR_PPERF, 0, 0, EVENT_OPTION_NONE_MASK},
 };
 
 
 static BoxMap skylake_box_map[NUM_UNITS] = {
     [PMC] = {MSR_PERF_GLOBAL_CTRL, MSR_V4_PERF_GLOBAL_STATUS, MSR_V4_PERF_GLOBAL_STATUS_RESET, 0, 0, 0, 48},
+    [FIXED] =  {MSR_PERF_GLOBAL_CTRL, MSR_V4_PERF_GLOBAL_STATUS, MSR_V4_PERF_GLOBAL_STATUS_RESET, 0, 0, 0, 48},
+    [PERF]    = {0, 0, 0, 0, 0, 0, 64},
     [THERMAL] = {0, 0, 0, 0, 0, 0, 8},
     [VOLTAGE] = {0, 0, 0, 0, 0, 0, 16},
-    [FIXED] =  {MSR_PERF_GLOBAL_CTRL, MSR_V4_PERF_GLOBAL_STATUS, MSR_V4_PERF_GLOBAL_STATUS_RESET, 0, 0, 0, 48},
     [POWER] = {0, 0, 0, 0, 0, 0, 32},
     [UBOXFIX] = {MSR_V4_UNC_PERF_GLOBAL_CTRL, MSR_V4_UNC_PERF_GLOBAL_STATUS, MSR_V4_UNC_PERF_GLOBAL_STATUS, 0, 0, 0, 44},
     [UBOX] = {MSR_V4_UNC_PERF_GLOBAL_CTRL, MSR_V4_UNC_PERF_GLOBAL_STATUS, MSR_V4_UNC_PERF_GLOBAL_STATUS, 1, 0, 0, 44},
@@ -103,3 +108,17 @@ static PciDevice skylake_pci_devices[MAX_NUM_PCI_DEVICES] = {
  [MSR_DEV] = {NODEVTYPE, "", "MSR", ""},
  [PCI_IMC_DEVICE_0_CH_0] = {IMC, "00.0", "MMAP_IMC_DEVICE", "MBOX0", 0x0},
 };
+
+static char* skylake_translate_types[NUM_UNITS] = {
+    [FIXED] = "/sys/bus/event_source/devices/cpu",
+    [PERF] = "/sys/bus/event_source/devices/msr",
+    [PMC] = "/sys/bus/event_source/devices/cpu",
+    [MBOX0] = "/sys/bus/event_source/devices/uncore_imc",
+    [CBOX0] = "/sys/bus/event_source/devices/uncore_cbox_0",
+    [CBOX1] = "/sys/bus/event_source/devices/uncore_cbox_1",
+    [CBOX2] = "/sys/bus/event_source/devices/uncore_cbox_2",
+    [CBOX3] = "/sys/bus/event_source/devices/uncore_cbox_3",
+    [UBOX] = "/sys/bus/event_source/devices/uncore_arb",
+    [UBOXFIX] = "/sys/bus/event_source/devices/uncore_arb",
+    [POWER] = "/sys/bus/event_source/devices/power",
+};
