@@ -5,14 +5,14 @@
  *
  *      Description:  Header File of registers.
  *
- *      Version:   5.1.0
- *      Released:  20.11.2020
+ *      Version:   5.2.2
+ *      Released:  26.07.2022
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2022 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -93,22 +93,34 @@ typedef enum {
     PMC331, PMC332, PMC333, PMC334, PMC335, PMC336,
     PMC337, PMC338, PMC339, PMC340, PMC341, PMC342,
     PMC343, PMC344, PMC345, PMC346, PMC347, PMC348,
+    PMC349, PMC350, PMC351, PMC352, PMC353, PMC354,
+    PMC355, PMC356, PMC357, PMC358, PMC359,
+    PMC360, PMC361, PMC362, PMC363, PMC364, PMC365, PMC366, PMC367, PMC368, PMC369,
+    PMC370, PMC371, PMC372, PMC373, PMC374, PMC375, PMC376, PMC377, PMC378, PMC379,
+    PMC380, PMC381, PMC382, PMC383, PMC384, PMC385, PMC386, PMC387, PMC388, PMC389,
+    PMC390, PMC391, PMC392, PMC393, PMC394, PMC395, PMC396, PMC397, PMC398, PMC399, 
+    PMC400, PMC401, PMC402, PMC403, PMC404, PMC405, PMC406, PMC407, PMC408, PMC409, 
     NUM_PMC
 } RegisterIndex;
 
 typedef enum {
-    PMC = 0, FIXED, THERMAL, VOLTAGE, METRICS,
+    PMC = 0, FIXED, PERF, THERMAL, VOLTAGE, METRICS,
     POWER, UNCORE, MBOX0,
     MBOX1, MBOX2, MBOX3,
     MBOX4, MBOX5, MBOX6, MBOX7,
+    MBOX8, MBOX9, MBOX10, MBOX11,
+    MBOX12,
     MBOX0FIX, MBOX1FIX, MBOX2FIX,
     MBOX3FIX, MBOX4FIX, MBOX5FIX,
-    MBOX6FIX, MBOX7FIX,
+    MBOX6FIX, MBOX7FIX, MBOX8FIX,
+    MBOX9FIX, MBOX10FIX, MBOX11FIX,
+    MBOX12FIX,
+    MDEV0, MDEV1, MDEV2, MDEV3,
     MBOX0TMP,
-    BBOX0, BBOX1,
+    BBOX0, BBOX1, BBOX2, BBOX3,
     RBOX0, RBOX1, RBOX2,
     WBOX,
-    WBOX0FIX, WBOX1FIX,
+    WBOX0FIX, WBOX1FIX, WBOX2FIX, WBOX3FIX,
     SBOX0, SBOX1, SBOX2, SBOX3,
     SBOX0FIX, SBOX1FIX, SBOX2FIX, SBOX3FIX,
     CBOX0, CBOX1, CBOX2,
@@ -123,8 +135,8 @@ typedef enum {
     CBOX27, CBOX28, CBOX29,
     CBOX30, CBOX31, CBOX32,
     CBOX33, CBOX34, CBOX35,
-    CBOX36, CBOX37,
-    PBOX, PBOX1, PBOX2, PBOX3,
+    CBOX36, CBOX37, CBOX38, CBOX39,
+    PBOX, PBOX1, PBOX2, PBOX3, PBOX4, PBOX5,
     UBOX,
     UBOXFIX,
     IBOX0, IBOX1, IBOX2, IBOX3, IBOX4, IBOX5,
@@ -145,6 +157,7 @@ typedef enum {
 static char* RegisterTypeNames[MAX_UNITS] = {
     [PMC] = "Core-local general purpose counters",
     [FIXED] = "Fixed counters",
+    [PERF] = "Perf counters",
     [THERMAL] = "Thermal",
     [VOLTAGE] = "Voltage of hardware thread",
     [METRICS] = "Performance metrics provided by Intel systems starting with Intel Icelake",
@@ -180,6 +193,8 @@ static char* RegisterTypeNames[MAX_UNITS] = {
     [WBOX] = "Power control box",
     [WBOX0FIX] = "Power control box fixed counter 0",
     [WBOX1FIX] = "Power control box fixed counter 1",
+    [WBOX2FIX] = "Power control box fixed counter 2",
+    [WBOX3FIX] = "Power control box fixed counter 3",
 #ifdef _ARCH_PPC
     [SBOX0] = "PowerBus",
 #else
@@ -230,10 +245,14 @@ static char* RegisterTypeNames[MAX_UNITS] = {
     [CBOX35] = "Caching Agent box 35",
     [CBOX36] = "Caching Agent box 36",
     [CBOX37] = "Caching Agent box 37",
-    [PBOX] = "Physical Layer box",
-    [PBOX1] = "Physical Layer box",
-    [PBOX2] = "Physical Layer box",
-    [PBOX3] = "Physical Layer box",
+    [CBOX38] = "Caching Agent box 38",
+    [CBOX39] = "Caching Agent box 39",
+    [PBOX] = "Physical Layer box 0",
+    [PBOX1] = "Physical Layer box 1",
+    [PBOX2] = "Physical Layer box 2",
+    [PBOX3] = "Physical Layer box 3",
+    [PBOX4] = "Physical Layer box 4",
+    [PBOX5] = "Physical Layer box 5",
     [UBOX] = "System Configuration box",
     [UBOXFIX] = "System Configuration box fixed counter",
     [IBOX0] = "Coherency Maintainer for IIO traffic",
@@ -324,7 +343,7 @@ static char* RegisterTypeNames[MAX_UNITS] = {
 #define MEASURE_METRICS(eventset) ((eventset)->regTypeMask1 & (REG_TYPE_MASK(METRICS))
 
 #define MEASURE_UNCORE(eventset) \
-        (eventset->regTypeMask1 & ~(REG_TYPE_MASK(PMC)|REG_TYPE_MASK(FIXED)|REG_TYPE_MASK(THERMAL)|REG_TYPE_MASK(VOLTAGE)|REG_TYPE_MASK(POWER)|REG_TYPE_MASK(METRICS)) || eventset->regTypeMask2 || eventset->regTypeMask3 || eventset->regTypeMask4)
+        (eventset->regTypeMask1 & ~(REG_TYPE_MASK(PMC)|REG_TYPE_MASK(FIXED)|REG_TYPE_MASK(THERMAL)|REG_TYPE_MASK(VOLTAGE)|REG_TYPE_MASK(PERF)|REG_TYPE_MASK(POWER)|REG_TYPE_MASK(METRICS)) || eventset->regTypeMask2 || eventset->regTypeMask3 || eventset->regTypeMask4)
 
 
 typedef struct {
