@@ -5,14 +5,14 @@
  *
  *      Description:  Header File of topology module.
  *
- *      Version:   5.1.0
- *      Released:  20.11.2020
+ *      Version:   5.2.2
+ *      Released:  26.07.2022
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2022 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -105,6 +105,7 @@ struct topology_functions {
 #define CANNONLAKE           0x66U
 #define ICELAKE1             0x7EU
 #define ICELAKE2             0x7DU
+#define ROCKETLAKE           0xA7U
 #define COMETLAKE1           0xA5U
 #define COMETLAKE2           0xA6U
 #define ICELAKEX1            0x6AU
@@ -141,8 +142,10 @@ struct topology_functions {
 #define ZENPLUS_RYZEN2  0x18
 #define ZEN2_RYZEN      0x31
 #define ZEN2_RYZEN2     0x71
+#define ZEN2_RYZEN3     0x60
 #define ZEN3_RYZEN      0x01
 #define ZEN3_RYZEN2     0x21
+#define ZEN3_RYZEN3     0x50
 
 /* ARM */
 #define  ARM7L          0x3U
@@ -197,6 +200,7 @@ struct topology_functions {
 
 extern int cpu_count(cpu_set_t* set);
 extern int likwid_cpu_online(int cpu_id);
+extern int likwid_sysfs_list_len(char* sysfsfile);
 
 static inline int cpuid_hasFeature(FeatureBit bit)
 {
