@@ -5,13 +5,13 @@
  *
  *      Description:  Marker API interface of module nvmon
  *
- *      Version:   5.1.0
- *      Released:  20.11.2020
+ *      Version:   5.2.2
+ *      Released:  26.07.2022
  *
  *      Authors:  Thomas Gruber (tg), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2022 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -460,22 +460,65 @@ likwid_gpuMarkerGetRegion(
         int* nr_gpus,
         int* nr_events,
         double** events,
-        double** time,
-        int **count)
+        double* time,
+        int *count)
 {
     if (!likwid_gpu_init)
     {
         *nr_gpus = 0;
         *nr_events = 0;
-        *time = NULL;
-        *events = NULL;
-        *count = NULL;
         return;
     }
     if (gettid() != main_tid)
     {
+        *nr_gpus = 0;
+        *nr_events = 0;
         return;
     }
+    bstring tag = bformat("%s-%d", regionTag, activeGpuGroup);
+    if (count != NULL)
+    {
+        for (int i = 0; i < MIN(nvmon_getNumberOfGPUs(), *nr_gpus); i++)
+        {
+            LikwidGpuResults *results = NULL;
+            int ret = get_smap_by_key(gpu_maps[i], bdata(tag), (void**)&results);
+            if (ret == 0)
+            {
+                count[i] = results->count;
+            }
+        }
+    }
+    if (time != NULL)
+    {
+        for (int i = 0; i < MIN(nvmon_getNumberOfGPUs(), *nr_gpus); i++)
+        {
+            LikwidGpuResults *results = NULL;
+            int ret = get_smap_by_key(gpu_maps[i], bdata(tag), (void**)&results);
+            if (ret == 0)
+            {
+                time[i] = results->time;
+            }
+        }
+    }
+    if (nr_events != NULL && events != NULL && *nr_events > 0)
+    {
+        for (int i = 0; i < MIN(nvmon_getNumberOfGPUs(), *nr_gpus); i++)
+        {
+            LikwidGpuResults *results = NULL;
+            int ret = get_smap_by_key(gpu_maps[i], bdata(tag), (void**)&results);
+            if (ret == 0)
+            {
+                for (int j = 0; j < MIN(nvmon_getNumberOfEvents(activeGpuGroup), *nr_events); j++)
+                {
+                    events[i][j] = results->PMcounters[j];
+                }
+            }
+        }
+        *nr_events = MIN(nvmon_getNumberOfEvents(activeGpuGroup), *nr_events);
+    }
+    *nr_gpus = MIN(nvmon_getNumberOfGPUs(), *nr_gpus);
+    bdestroy(tag);
+    return;
 }
 
 
@@ -495,7 +538,7 @@ likwid_gpuMarkerResetRegion(const char* regionTag)
     {
         LikwidGpuResults *results = NULL;
         int ret = get_smap_by_key(gpu_maps[i], bdata(tag), (void**)&results);
-        if ((ret < 0) || (results->state != GPUMARKER_STATE_START))
+        if ((ret < 0) || (results->state != GPUMARKER_STATE_STOP))
         {
             fprintf(stderr, "ERROR: Can only reset known/stopped regions\n");
             return -EFAULT;
