@@ -5,14 +5,14 @@
  *
  *      Description:  Marker API interface of module perfmon
  *
- *      Version:   5.0
- *      Released:  10.11.2019
+ *      Version:   5.2.2
+ *      Released:  26.07.2022
  *
  *      Authors:  Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2019 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2022 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -54,6 +54,7 @@
 #include <affinity.h>
 #include <perfmon.h>
 #include <bstrlib.h>
+#include <voltage.h>
 
 /* #####   VARIABLES  -  LOCAL TO THIS SOURCE FILE   ###################### */
 
@@ -112,6 +113,10 @@ calculateMarkerResult(RegisterIndex index, uint64_t start, uint64_t stop, int ov
 {
     double result = 0.0;
     uint64_t maxValue = 0ULL;
+    if (start > stop)
+    {
+        overflows++;
+    }
     if (overflows == 0)
     {
         result = (double) (stop - start);
@@ -129,10 +134,15 @@ calculateMarkerResult(RegisterIndex index, uint64_t start, uint64_t stop, int ov
     {
         result *= power_getEnergyUnit(getCounterTypeOffset(index));
     }
-    else if (counter_map[index].type == THERMAL)
+    else if ((counter_map[index].type == THERMAL) ||
+             (counter_map[index].type == MBOX0TMP))
     {
         result = (double)stop;
     }
+    else if (counter_map[index].type == VOLTAGE)
+    {
+        result = voltage_value(stop);
+    }
     return result;
 }
 
@@ -484,8 +494,9 @@ likwid_markerClose(void)
     {
         free(results);
     }
-    likwid_init = 0;
+    perfmon_finalize();
     HPMfinalize();
+    likwid_init = 0;
 }
 
 int
@@ -602,6 +613,10 @@ likwid_markerStopRegion(const char* regionTag)
     if (results->state != MARKER_STATE_START)
     {
         fprintf(stderr, "WARN: Stopping an unknown/not-started region %s\n", regionTag);
+        if (use_locks == 1)
+        {
+            pthread_mutex_lock(&threadLocks[myCPU]);
+        }
         return -EFAULT;
     }
     results->groupID = groupSet->activeGroup;
@@ -616,13 +631,15 @@ likwid_markerStopRegion(const char* regionTag)
     {
         if (groupSet->groups[groupSet->activeGroup].events[i].type != NOTYPE)
         {
-            DEBUG_PRINT(DEBUGLEV_DEVELOP, STOP [%s] READ EVENT [%d=%d] EVENT %d VALUE %llu, regionTag, thread_id, cpu_id, i,
-                            LLU_CAST groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].counterData);
             result = calculateMarkerResult(groupSet->groups[groupSet->activeGroup].events[i].index, results->StartPMcounters[i],
                                             groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].counterData,
                                             groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].overflows -
                                             results->StartOverflows[i]);
-            if (counter_map[groupSet->groups[groupSet->activeGroup].events[i].index].type != THERMAL)
+            DEBUG_PRINT(DEBUGLEV_DEVELOP, STOP [%s] READ EVENT [%d=%d] EVENT %d VALUE %llu DIFF %f, regionTag, thread_id, cpu_id, i,
+                            LLU_CAST groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].counterData, result);
+            if ((counter_map[groupSet->groups[groupSet->activeGroup].events[i].index].type != THERMAL) &&
+                (counter_map[groupSet->groups[groupSet->activeGroup].events[i].index].type != VOLTAGE) &&
+                (counter_map[groupSet->groups[groupSet->activeGroup].events[i].index].type != MBOX0TMP))
             {
                 results->PMcounters[i] += result;
             }
@@ -787,3 +804,4 @@ likwid_pinProcess(int processorId)
 
     return TRUE;
 }
+
