@@ -6,14 +6,14 @@
  *      Description:  Implementation of Linux NUMA interface. Selects between hwloc and
  *                    procfs/sysfs backends.
  *
- *      Version:   5.1.0
- *      Released:  20.11.2020
+ *      Version:   5.2.2
+ *      Released:  26.07.2022
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2022 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -255,9 +255,9 @@ numa_init(void)
     }
     else
     {
-        cpu_set_t cpuSet;
-        CPU_ZERO(&cpuSet);
-        sched_getaffinity(0,sizeof(cpu_set_t), &cpuSet);
+/*        cpu_set_t cpuSet;*/
+/*        CPU_ZERO(&cpuSet);*/
+/*        sched_getaffinity(0,sizeof(cpu_set_t), &cpuSet);*/
         if (cpuid_topology.activeHWThreads < cpuid_topology.numHWThreads &&
             getenv("HWLOC_FSROOT") == NULL)
         {
