@@ -5,14 +5,14 @@
  *
  *      Description:  Module implementing Intel RAPL interface
  *
- *      Version:   5.1.0
- *      Released:  20.11.2020
+ *      Version:   5.2.2
+ *      Released:  26.07.2022
  *
  *      Authors:  Jan Treibig (jt), jan.treibig@gmail.com,
  *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2022 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -56,6 +56,7 @@ power_init(int cpuId)
     uint64_t flags;
     int i;
     int err;
+    int core_limits = 0;
     uint32_t unit_reg = MSR_RAPL_POWER_UNIT;
     int numDomains = NUM_POWER_DOMAINS;
     Configuration_t config;
@@ -121,16 +122,23 @@ power_init(int cpuId)
                     break;
                 case SKYLAKE1:
                 case SKYLAKE2:
-                case SKYLAKEX:
                 case KABYLAKE1:
                 case KABYLAKE2:
                 case CANNONLAKE:
+                case COMETLAKE1:
+                case COMETLAKE2:
                 case TIGERLAKE1:
                 case TIGERLAKE2:
                 case ICELAKE1:
                 case ICELAKE2:
+                case ROCKETLAKE:
+                    power_info.hasRAPL = 1;
+                    numDomains = NUM_POWER_DOMAINS;
+                    break;
+                case SKYLAKEX:
                 case ICELAKEX1:
                 case ICELAKEX2:
+                    core_limits = 1;
                     power_info.hasRAPL = 1;
                     numDomains = NUM_POWER_DOMAINS;
                     break;
@@ -155,8 +163,10 @@ power_init(int cpuId)
                 cpuid_info.model == ZENPLUS_RYZEN2 ||
                 cpuid_info.model == ZEN2_RYZEN ||
                 cpuid_info.model == ZEN2_RYZEN2 ||
+                cpuid_info.model == ZEN2_RYZEN3 ||
                 cpuid_info.model == ZEN3_RYZEN ||
-                cpuid_info.model == ZEN3_RYZEN2)
+                cpuid_info.model == ZEN3_RYZEN2 ||
+                cpuid_info.model == ZEN3_RYZEN3)
             {
                 cpuid_info.turbo = 0;
                 power_info.hasRAPL = 1;
@@ -226,7 +236,7 @@ power_init(int cpuId)
             }
             else
             {
-                if (cpuid_info.model != SKYLAKEX)
+                if (!core_limits)
                 {
                     unsigned long flag_vals[4];
                     int flag_idx = 0;
@@ -337,6 +347,8 @@ power_init(int cpuId)
                 (cpuid_info.model == BROADWELL_D) ||
                 (cpuid_info.model == BROADWELL_E) ||
                 (cpuid_info.model == SKYLAKEX) ||
+                (cpuid_info.model == ICELAKEX1) ||
+                (cpuid_info.model == ICELAKEX2) ||
                 (cpuid_info.model == XEON_PHI_KNL) ||
                 (cpuid_info.model == XEON_PHI_KML))
             {
@@ -429,8 +441,8 @@ power_init(int cpuId)
             err = HPMread(cpuId, MSR_DEV, MSR_UNCORE_FREQ, &flags);
             if (err == 0)
             {
-                power_info.uncoreMinFreq = ((double)((flags >> 8) & 0xFFULL)) * busSpeed;
-                power_info.uncoreMaxFreq = ((double)(flags & 0xFF)) * busSpeed;
+                power_info.uncoreMinFreq = ((double)((flags >> 8) & 0x3FULL)) * busSpeed;
+                power_info.uncoreMaxFreq = ((double)(flags & 0x3F)) * busSpeed;
             }
             err = HPMread(cpuId, MSR_DEV, MSR_ENERGY_PERF_BIAS, &flags);
             if (err == 0)
