@@ -5,14 +5,14 @@
  *
  *      Description:  Interface to the topology backends
  *
- *      Version:   5.1.0
- *      Released:  20.11.2020
+ *      Version:   5.2.2
+ *      Released:  26.07.2022
  *
  *      Authors:  Jan Treibig (jt), jan.treibig@gmail.com,
  *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2022 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -45,7 +45,7 @@
 #include <bitUtil.h>
 //#include <strUtil.h>
 #include <configuration.h>
-#include <topology_cavtx2.h>
+#include <topology_static.h>
 
 /* #####   VARIABLES  -  LOCAL TO THIS SOURCE FILE   ###################### */
 
@@ -82,6 +82,7 @@ static char* cascadelakeX_str = "Intel Cascadelake SP processor";
 static char* kabylake_str = "Intel Kabylake processor";
 static char* cannonlake_str = "Intel Cannonlake processor";
 static char* coffeelake_str = "Intel Coffeelake processor";
+static char* cometlake_str = "Intel Cometlake processor";
 static char* nehalem_ex_str = "Intel Nehalem EX processor";
 static char* westmere_ex_str = "Intel Westmere EX processor";
 static char* xeon_mp_string = "Intel Xeon MP processor";
@@ -91,6 +92,7 @@ static char* xeon_phi3_string = "Intel Xeon Phi (Knights Mill) (Co)Processor";
 static char* icelake_str = "Intel Icelake processor";
 static char* tigerlake_str = "Intel Tigerlake processor";
 static char* icelakesp_str = "Intel Icelake SP processor";
+static char* rocketlake_str = "Intel Rocketlake processor";
 //static char* snowridgex_str = "Intel SnowridgeX processor";
 
 static char* barcelona_str = "AMD K10 (Barcelona) processor";
@@ -116,8 +118,10 @@ static char* armv7l_str = "ARM 7l architecture";
 static char* armv8_str = "ARM 8 architecture";
 static char* cavium_thunderx2t99_str = "Cavium Thunder X2 (ARMv8)";
 static char* cavium_thunderx_str = "Cavium Thunder X (ARMv8)";
-static char* arm_cortex_a57 = "ARM Cortex A57 (ARMv8)";
-static char* arm_cortex_a53 = "ARM Cortex A53 (ARMv8)";
+static char* arm_cortex_a57 = "ARM Cortex A57";
+static char* arm_cortex_a53 = "ARM Cortex A53";
+static char* arm_cortex_a72 = "ARM Cortex A72";
+static char* arm_cortex_a73 = "ARM Cortex A73";
 static char* arm_neoverse_n1 = "ARM Neoverse N1";
 static char* fujitsu_a64fx = "Fujitsu A64FX";
 static char* power7_str = "POWER7 architecture";
@@ -156,6 +160,7 @@ static char* short_tigerlake = "TGL";
 static char* short_phi = "phi";
 static char* short_phi2 = "knl";
 static char* short_icelake = "ICL";
+static char* short_rocketlake = "RKL";
 static char* short_icelakesp = "ICX";
 //static char* short_snowridgex = "SNR";
 
@@ -252,15 +257,15 @@ readTopologyFile(const char* filename, cpu_set_t cpuSet)
 
     while (fgets(line, 512, fp) != NULL) {
         sscanf(line,"%s %s", structure, field);
-        if ((strcmp(structure, "cpuid_topology") == 0) && (strcmp(field, "numHWThreads") == 0))
+        if ((strncmp(structure, "cpuid_topology", 14) == 0) && (strncmp(field, "numHWThreads", 12) == 0))
         {
             sscanf(line,"%s %s = %d", structure, field, &numHWThreads);
         }
-        else if ((strcmp(structure, "cpuid_topology") == 0) && (strcmp(field, "numCacheLevels") == 0))
+        else if ((strncmp(structure, "cpuid_topology", 14) == 0) && (strncmp(field, "numCacheLevels", 15) == 0))
         {
             sscanf(line,"%s %s = %d", structure, field, &numCacheLevels);
         }
-        else if ((strcmp(structure, "numa_info") == 0) && (strcmp(field, "numberOfNodes") == 0))
+        else if ((strncmp(structure, "numa_info", 9) == 0) && (strncmp(field, "numberOfNodes", 13) == 0))
         {
             sscanf(line,"%s %s = %d", structure, field, &numberOfNodes);
         }
@@ -281,7 +286,7 @@ readTopologyFile(const char* filename, cpu_set_t cpuSet)
     counter = 0;
     while (fgets(line, 512, fp) != NULL) {
         sscanf(line,"%s %s %d %s = %d", structure, field, &tmp, value, &tmp1);
-        if ((strcmp(structure, "numa_info") == 0) && (strcmp(value, "numberOfProcessors") == 0))
+        if ((strncmp(structure, "numa_info", 9) == 0) && (strncmp(value, "numberOfProcessors", 19) == 0))
         {
             tmpNumberOfProcessors[tmp-1] = tmp1;
             counter++;
@@ -311,42 +316,51 @@ readTopologyFile(const char* filename, cpu_set_t cpuSet)
 
     while (fgets(line, 512, fp) != NULL) {
         sscanf(line,"%s %s", structure, field);
-        if (strcmp(structure, "cpuid_topology") == 0)
+        if (strncmp(structure, "cpuid_topology", 14) == 0)
         {
-            if (strcmp(field, "numSockets") == 0)
+            if (strncmp(field, "numSockets", 11) == 0)
             {
                 sscanf(line, "%s %s = %d", structure, field, &tmp);
                 cpuid_topology.numSockets = tmp;
             }
-            else if (strcmp(field, "numCoresPerSocket") == 0)
+            else if (strncmp(field, "numCoresPerSocket", 18) == 0)
             {
                 sscanf(line, "%s %s = %d", structure, field, &tmp);
                 cpuid_topology.numCoresPerSocket = tmp;
             }
-            else if (strcmp(field, "numThreadsPerCore") == 0)
+            else if (strncmp(field, "numThreadsPerCore", 18) == 0)
             {
                 sscanf(line, "%s %s = %d", structure, field, &tmp);
                 cpuid_topology.numThreadsPerCore = tmp;
             }
-            else if (strcmp(field, "threadPool") == 0)
+            else if (strncmp(field, "numDies", 7) == 0)
+            {
+                sscanf(line, "%s %s = %d", structure, field, &tmp);
+                cpuid_topology.numDies = tmp;
+            }
+            else if (strncmp(field, "threadPool", 11) == 0)
             {
                 int thread;
 
                 sscanf(line, "%s %s %d %s = %d", structure, field, &thread, value, &tmp);
 
-                if (strcmp(value, "threadId") == 0)
+                if (strncmp(value, "threadId", 8) == 0)
                 {
                     cpuid_topology.threadPool[thread].threadId = tmp;
                 }
-                else if (strcmp(value, "coreId") == 0)
+                else if (strncmp(value, "coreId", 6) == 0)
                 {
                     cpuid_topology.threadPool[thread].coreId = tmp;
                 }
-                else if (strcmp(value, "packageId") == 0)
+                else if (strncmp(value, "packageId", 10) == 0)
                 {
                     cpuid_topology.threadPool[thread].packageId = tmp;
                 }
-                else if (strcmp(value, "apicId") == 0)
+                else if (strncmp(value, "dieId", 5) == 0)
+                {
+                    cpuid_topology.threadPool[thread].dieId = tmp;
+                }
+                else if (strncmp(value, "apicId", 6) == 0)
                 {
                     cpuid_topology.threadPool[thread].apicId = tmp;
                     if (CPU_ISSET(tmp, &cpuSet))
@@ -360,37 +374,37 @@ readTopologyFile(const char* filename, cpu_set_t cpuSet)
                 }
 
             }
-            else if (strcmp(field, "cacheLevels") == 0)
+            else if (strncmp(field, "cacheLevels", 12) == 0)
             {
                 int level;
                 char type[128];
                 sscanf(line, "%s %s %d %s", structure, field, &level, value);
 
                 cpuid_topology.cacheLevels[level-1].level = level-1;
-                if (strcmp(value, "type") == 0)
+                if (strncmp(value, "type", 4) == 0)
                 {
                     sscanf(line, "%s %s %d %s = %s", structure, field, &level, value, type);
-                    if (strcmp(type, "UNIFIEDCACHE") == 0)
+                    if (strncmp(type, "UNIFIEDCACHE", 13) == 0)
                     {
                         cpuid_topology.cacheLevels[level-1].type = UNIFIEDCACHE;
                     }
-                    else if (strcmp(type, "DATACACHE") == 0)
+                    else if (strncmp(type, "DATACACHE", 9) == 0)
                     {
                         cpuid_topology.cacheLevels[level-1].type = DATACACHE;
                     }
-                    else if (strcmp(type, "INSTRUCTIONCACHE") == 0)
+                    else if (strncmp(type, "INSTRUCTIONCACHE", 17) == 0)
                     {
                         cpuid_topology.cacheLevels[level-1].type = INSTRUCTIONCACHE;
                     }
-                    else if (strcmp(type, "ITLB") == 0)
+                    else if (strncmp(type, "ITLB", 4) == 0)
                     {
                         cpuid_topology.cacheLevels[level-1].type = ITLB;
                     }
-                    else if (strcmp(type, "DTLB") == 0)
+                    else if (strncmp(type, "DTLB", 4) == 0)
                     {
                         cpuid_topology.cacheLevels[level-1].type = DTLB;
                     }
-                    else if (strcmp(type, "NOCACHE") == 0)
+                    else if (strncmp(type, "NOCACHE", 7) == 0)
                     {
                         cpuid_topology.cacheLevels[level-1].type = NOCACHE;
                     }
@@ -398,27 +412,27 @@ readTopologyFile(const char* filename, cpu_set_t cpuSet)
                 else
                 {
                     sscanf(line, "%s %s %d %s = %d", structure, field, &level, value, &tmp);
-                    if (strcmp(value, "associativity") == 0)
+                    if (strncmp(value, "associativity", 14) == 0)
                     {
                         cpuid_topology.cacheLevels[level-1].associativity = tmp;
                     }
-                    else if (strcmp(value, "sets") == 0)
+                    else if (strncmp(value, "sets", 4) == 0)
                     {
                         cpuid_topology.cacheLevels[level-1].sets = tmp;
                     }
-                    else if (strcmp(value, "lineSize") == 0)
+                    else if (strncmp(value, "lineSize", 8) == 0)
                     {
                         cpuid_topology.cacheLevels[level-1].lineSize = tmp;
                     }
-                    else if (strcmp(value, "size") == 0)
+                    else if (strncmp(value, "size", 4) == 0)
                     {
                         cpuid_topology.cacheLevels[level-1].size = tmp;
                     }
-                    else if (strcmp(value, "threads") == 0)
+                    else if (strncmp(value, "threads", 9) == 0)
                     {
                         cpuid_topology.cacheLevels[level-1].threads = tmp;
                     }
-                    else if (strcmp(value, "inclusive") == 0)
+                    else if (strncmp(value, "inclusive", 10) == 0)
                     {
                         cpuid_topology.cacheLevels[level-1].inclusive = tmp;
                     }
@@ -426,20 +440,20 @@ readTopologyFile(const char* filename, cpu_set_t cpuSet)
 
             }
         }
-        else if (strcmp(structure, "cpuid_info") == 0)
+        else if (strncmp(structure, "cpuid_info", 11) == 0)
         {
-            if (strcmp(field, "family") == 0)
+            if (strncmp(field, "family", 6) == 0)
             {
                 sscanf(line, "%s %s = %d", structure, field, &tmp);
                 cpuid_info.family = tmp;
 
             }
-            else if (strcmp(field, "model") == 0)
+            else if (strncmp(field, "model", 5) == 0)
             {
                 sscanf(line, "%s %s = %d", structure, field, &tmp);
                 cpuid_info.model = tmp;
             }
-            else if (strcmp(field, "osname") == 0)
+            else if (strncmp(field, "osname", 6) == 0)
             {
                 strncpy(value,&(line[strlen(structure)+strlen(field)+4]), 256);
                 int len = 257;
@@ -447,107 +461,120 @@ readTopologyFile(const char* filename, cpu_set_t cpuSet)
                 strncpy(cpuid_info.osname, value, len);
                 cpuid_info.osname[strlen(value)-1] = '\0';
             }
-            else if (strcmp(field, "stepping") == 0)
+            else if (strncmp(field, "stepping", 9) == 0)
             {
                 sscanf(line, "%s %s = %d", structure, field, &tmp);
                 cpuid_info.stepping = tmp;
-
             }
-            else if (strcmp(field, "clock") == 0)
+            else if (strncmp(field, "vendor", 6) == 0)
+            {
+                sscanf(line, "%s %s = %d", structure, field, &tmp);
+                cpuid_info.vendor = tmp;
+            }
+            else if (strncmp(field, "part", 4) == 0)
+            {
+                sscanf(line, "%s %s = %d", structure, field, &tmp);
+                cpuid_info.part = tmp;
+            }
+            else if (strncmp(field, "clock", 5) == 0)
             {
                 sscanf(line, "%s %s = %d", structure, field, &tmp);
                 cpuid_info.clock = tmp;
-
             }
-            else if (strcmp(field, "turbo") == 0)
+            else if (strncmp(field, "turbo", 5) == 0)
             {
                 sscanf(line, "%s %s = %d", structure, field, &tmp);
                 cpuid_info.turbo = tmp;
-
             }
-            else if (strcmp(field, "isIntel") == 0)
+            else if (strncmp(field, "isIntel", 7) == 0)
             {
                 sscanf(line, "%s %s = %d", structure, field, &tmp);
                 cpuid_info.isIntel = tmp;
-
             }
-            else if (strcmp(field, "featureFlags") == 0)
+            else if (strncmp(field, "featureFlags", 13) == 0)
             {
                 sscanf(line, "%s %s = %d", structure, field, &tmp);
                 cpuid_info.featureFlags = tmp;
 
             }
-            else if (strcmp(field, "perf_version") == 0)
+            else if (strncmp(field, "perf_version", 13) == 0)
             {
                 sscanf(line, "%s %s = %d", structure, field, &tmp);
                 cpuid_info.perf_version = tmp;
 
             }
-            else if (strcmp(field, "perf_num_ctr") == 0)
+            else if (strncmp(field, "perf_num_ctr", 13) == 0)
             {
                 sscanf(line, "%s %s = %d", structure, field, &tmp);
                 cpuid_info.perf_num_ctr = tmp;
-
             }
-            else if (strcmp(field, "perf_width_ctr") == 0)
+            else if (strncmp(field, "perf_width_ctr", 15) == 0)
             {
                 sscanf(line, "%s %s = %d", structure, field, &tmp);
                 cpuid_info.perf_width_ctr = tmp;
-
             }
-            else if (strcmp(field, "perf_num_fixed_ctr") == 0)
+            else if (strncmp(field, "perf_num_fixed_ctr", 19) == 0)
             {
                 sscanf(line, "%s %s = %d", structure, field, &tmp);
                 cpuid_info.perf_num_fixed_ctr = tmp;
-
             }
-            else if (strcmp(field, "features") == 0)
+            else if (strncmp(field, "supportClientmem", 17) == 0)
+            {
+                sscanf(line, "%s %s = %d", structure, field, &tmp);
+                cpuid_info.supportClientmem = tmp;
+            }
+            else if (strncmp(field, "supportUncore", 14) == 0)
+            {
+                sscanf(line, "%s %s = %d", structure, field, &tmp);
+                cpuid_info.supportUncore = tmp;
+            }
+            else if (strncmp(field, "features", 9) == 0)
             {
                 strcpy(value,&(line[strlen(structure)+strlen(field)+4]));
-		int len = 257;
+                int len = 257;
                 cpuid_info.features = (char*) malloc(len * sizeof(char));
                 strncpy(cpuid_info.features, value, len);
                 cpuid_info.features[strlen(value)-1] = '\0';
             }
         }
-        else if (strcmp(structure, "numa_info") == 0)
+        else if (strncmp(structure, "numa_info", 9) == 0)
         {
-            if (strcmp(field, "nodes") == 0)
+            if (strncmp(field, "nodes", 5) == 0)
             {
                 int id;
                 sscanf(line, "%s %s %d %s", structure, field, &id, value);
 
-                if (strcmp(value,"numberOfProcessors") == 0)
+                if (strncmp(value,"numberOfProcessors", 19) == 0)
                 {
                     sscanf(line, "%s %s %d %s = %d", structure, field, &id, value, &tmp);
                     numa_info.nodes[id-1].numberOfProcessors = tmp;
                 }
-                else if (strcmp(value, "freeMemory") == 0)
+                else if (strncmp(value, "freeMemory", 11) == 0)
                 {
                     sscanf(line, "%s %s %d %s = %d", structure, field, &id, value, &tmp);
                     numa_info.nodes[id-1].freeMemory = tmp;
                 }
-                else if (strcmp(value, "id") == 0)
+                else if (strncmp(value, "id", 2) == 0)
                 {
                     sscanf(line, "%s %s %d %s = %d", structure, field, &id, value, &tmp);
                     numa_info.nodes[id-1].id = tmp;
                 }
-                else if (strcmp(value, "totalMemory") == 0)
+                else if (strncmp(value, "totalMemory", 12) == 0)
                 {
                     sscanf(line, "%s %s %d %s = %d", structure, field, &id, value, &tmp);
                     numa_info.nodes[id-1].totalMemory = tmp;
                 }
-                else if (strcmp(value, "numberOfDistances") == 0)
+                else if (strncmp(value, "numberOfDistances", 18) == 0)
                 {
                     sscanf(line, "%s %s %d %s = %d", structure, field, &id, value, &tmp);
                     numa_info.nodes[id-1].numberOfDistances = tmp;
                 }
-                if (strcmp(value, "processors") == 0)
+                if (strncmp(value, "processors", 11) == 0)
                 {
                     sscanf(line, "%s %s %d %s %d = %d", structure, field, &id, value, &tmp, &tmp1);
                     numa_info.nodes[id-1].processors[tmp-1] = tmp1;
                 }
-                else if (strcmp(value,"distances") == 0)
+                else if (strncmp(value,"distances", 10) == 0)
                 {
                     sscanf(line, "%s %s %d %s %d = %d", structure, field, &id, value, &tmp, &tmp1);
                     numa_info.nodes[id-1].distances[tmp] = tmp1;
@@ -649,6 +676,38 @@ likwid_cpu_online(int cpu_id)
     return state;
 }
 
+int likwid_sysfs_list_len(char* sysfsfile)
+{
+    int i = 0;
+    int len = 0;
+    FILE* fp;
+    if (NULL != (fp = fopen (sysfsfile, "r")))
+    {
+        bstring src = bread ((bNread) fread, fp);
+        struct bstrList* first = bsplit(src, ',');
+        for (i = 0; i < first->qty; i++)
+        {
+            struct bstrList* second = bsplit(first->entry[i], '-');
+            if (second->qty == 1)
+            {
+                len++;
+            }
+            else
+            {
+                int s = atoi(bdata(second->entry[0]));
+                int e = atoi(bdata(second->entry[1]));
+                len += e - s + 1;
+            }
+            bstrListDestroy(second);
+        }
+        bstrListDestroy(first);
+        bdestroy(src);
+        fclose(fp);
+    }
+    return len;
+}
+
+
 
 int
 topology_setName(void)
@@ -775,6 +834,7 @@ topology_setName(void)
                         cpuid_info.name = cascadelakeX_str;
                         cpuid_info.short_name = short_cascadelakeX;
                     }
+                    cpuid_info.supportUncore = 1;
                     break;
 
                 case KABYLAKE1:
@@ -795,6 +855,13 @@ topology_setName(void)
                     cpuid_info.short_name = short_cannonlake;
                     break;
 
+                case COMETLAKE1:
+                case COMETLAKE2:
+                    cpuid_info.supportClientmem = 1;
+                    cpuid_info.name = cometlake_str;
+                    cpuid_info.short_name = short_skylake;
+                    break;
+
                 case XEON_PHI_KNL:
                     cpuid_info.supportUncore = 1;
                     cpuid_info.name = xeon_phi2_string;
@@ -872,8 +939,15 @@ topology_setName(void)
                     cpuid_info.short_name = short_icelake;
                     break;
 
+                case ROCKETLAKE:
+                    cpuid_info.supportClientmem = 1;
+                    cpuid_info.name = rocketlake_str;
+                    cpuid_info.short_name = short_rocketlake;
+                    break;
+
                 case ICELAKEX1:
                 case ICELAKEX2:
+                    cpuid_info.supportUncore = 1;
                     cpuid_info.name = icelakesp_str;
                     cpuid_info.short_name = short_icelakesp;
                     break;
@@ -998,7 +1072,7 @@ topology_setName(void)
             cpuid_info.short_name = short_k16;
             break;
 
-	case PPC_FAMILY:
+        case PPC_FAMILY:
             switch(cpuid_info.model)
             {
                 case POWER7:
@@ -1035,6 +1109,7 @@ topology_setName(void)
                     break;
                 case ZEN2_RYZEN:
                 case ZEN2_RYZEN2:
+                case ZEN2_RYZEN3:
                     cpuid_info.name = amd_zen2_str;
                     cpuid_info.short_name = short_zen2;
                     break;
@@ -1051,6 +1126,10 @@ topology_setName(void)
                     cpuid_info.name = amd_zen3_str;
                     cpuid_info.short_name = short_zen3;
                     break;
+                case ZEN3_RYZEN3:
+                    cpuid_info.name = amd_zen3_str;
+                    cpuid_info.short_name = short_zen3;
+                    break;
             }
             break;
 
@@ -1062,6 +1141,22 @@ topology_setName(void)
                     cpuid_info.name = armv7l_str;
                     cpuid_info.short_name = short_arm7;
                     break;
+                case ARM_CORTEX_A57:
+                    cpuid_info.name = arm_cortex_a57;
+                    cpuid_info.short_name = short_arm7;
+                    break;
+                case ARM_CORTEX_A53:
+                    cpuid_info.name = arm_cortex_a53;
+                    cpuid_info.short_name = short_arm7;
+                    break;
+                case ARM_CORTEX_A72:
+                    cpuid_info.name = arm_cortex_a72;
+                    cpuid_info.short_name = short_arm7;
+                    break;
+                case ARM_CORTEX_A73:
+                    cpuid_info.name = arm_cortex_a73;
+                    cpuid_info.short_name = short_arm7;
+                    break;
                 default:
                     return EXIT_FAILURE;
                     break;
@@ -1081,6 +1176,14 @@ topology_setName(void)
                             cpuid_info.name = arm_cortex_a53;
                             cpuid_info.short_name = short_arm8;
                             break;
+                        case ARM_CORTEX_A72:
+                            cpuid_info.name = arm_cortex_a72;
+                            cpuid_info.short_name = short_arm8;
+                            break;
+                        case ARM_CORTEX_A73:
+                            cpuid_info.name = arm_cortex_a73;
+                            cpuid_info.short_name = short_arm8;
+                            break;
                         case ARM_NEOVERSE_N1:
                             cpuid_info.name = arm_neoverse_n1;
                             cpuid_info.short_name = short_arm8_neo_n1;
@@ -1178,6 +1281,12 @@ void topology_setupTree(void)
         }
         currentNode = tree_getNode(cpuid_topology.topologyTree,
                 hwThreadPool[i].packageId);
+        /*if (!tree_nodeExists(currentNode, hwThreadPool[i].dieId))
+        {
+            printf("Insert Die %d at Socket %d\n", hwThreadPool[i].dieId, hwThreadPool[i].packageId);
+            tree_insertNode(currentNode, hwThreadPool[i].dieId);
+        }
+        currentNode = tree_getNode(currentNode, hwThreadPool[i].dieId);*/
         if (!tree_nodeExists(currentNode, hwThreadPool[i].coreId))
         {
             //printf("Insert Core %d at Socket %d\n", hwThreadPool[i].coreId, hwThreadPool[i].packageId);
@@ -1189,7 +1298,7 @@ void topology_setupTree(void)
             /*
                printf("WARNING: Thread already exists!\n");
                */
-            //printf("Insert HWThread %d from Core %d at Socket %d\n", hwThreadPool[i].apicId, hwThreadPool[i].coreId, hwThreadPool[i].packageId);
+            //printf("Insert HWThread %d at Core %d Socket %d\n", hwThreadPool[i].apicId, hwThreadPool[i].coreId, hwThreadPool[i].packageId);
             tree_insertNode(currentNode, hwThreadPool[i].apicId);
         }
 
@@ -1229,7 +1338,20 @@ topology_init(void)
     {
 standard_init:
         CPU_ZERO(&cpuSet);
-        sched_getaffinity(0,sizeof(cpu_set_t), &cpuSet);
+        if (getenv("LIKWID_IGNORE_CPUSET") == NULL)
+        {
+            sched_getaffinity(0,sizeof(cpu_set_t), &cpuSet);
+        }
+        else
+        {
+            for (int i = 0; i < sysconf(_SC_NPROCESSORS_CONF); i++)
+            {
+                if (likwid_cpu_online(i))
+                {
+                    CPU_SET(i, &cpuSet);
+                }
+            }
+        }
         if (cpu_count(&cpuSet) < sysconf(_SC_NPROCESSORS_CONF))
         {
 #if !defined(__ARM_ARCH_7A__) && !defined(__ARM_ARCH_8A)
@@ -1249,6 +1371,14 @@ standard_init:
         topology_setName();
         funcs.init_cpuFeatures();
         funcs.init_nodeTopology(cpuSet);
+        int activeCount = 0;
+        for (int i = 0; i < cpuid_topology.numHWThreads; i++)
+        {
+            if (cpuid_topology.threadPool[i].inCpuSet)
+                activeCount++;
+        }
+        if (activeCount > cpuid_topology.activeHWThreads)
+            cpuid_topology.activeHWThreads = activeCount;
         funcs.init_cacheTopology();
         if (cpuid_topology.numCacheLevels == 0)
         {
@@ -1299,6 +1429,27 @@ standard_init:
                                 default:
                                     break;
                             }
+                        case FUJITSU_ARM:
+                            switch(cpuid_info.part) {
+                                case FUJITSU_A64FX:
+                                    cachePool = (CacheLevel*) malloc(2 * sizeof(CacheLevel));
+                                    for(int i=0;i < 2; i++)
+                                    {
+                                        cachePool[i].level = a64fx_caches[i].level;
+                                        cachePool[i].size = a64fx_caches[i].size;
+                                        cachePool[i].lineSize = a64fx_caches[i].lineSize;
+                                        cachePool[i].threads = a64fx_caches[i].threads;
+                                        cachePool[i].inclusive = a64fx_caches[i].inclusive;
+                                        cachePool[i].sets = a64fx_caches[i].sets;
+                                        cachePool[i].associativity = a64fx_caches[i].associativity;
+                                    }
+                                    cpuid_topology.cacheLevels = cachePool;
+                                    cpuid_topology.numCacheLevels = 2;
+                                    break;
+                                default:
+                                    break;
+                            }
+                            break;
                         default:
                             break;
                     }
@@ -1313,7 +1464,20 @@ standard_init:
     else
     {
         CPU_ZERO(&cpuSet);
-        sched_getaffinity(0, sizeof(cpu_set_t), &cpuSet);
+        if (getenv("LIKWID_IGNORE_CPUSET") == NULL)
+        {
+            sched_getaffinity(0, sizeof(cpu_set_t), &cpuSet);
+        }
+        else
+        {
+            for (int i = 0; i < sysconf(_SC_NPROCESSORS_CONF); i++)
+            {
+                if (likwid_cpu_online(i))
+                {
+                    CPU_SET(i, &cpuSet);
+                }
+            }
+        }
         DEBUG_PRINT(DEBUGLEV_INFO, Reading topology information from %s, config.topologyCfgFileName);
         ret = readTopologyFile(config.topologyCfgFileName, cpuSet);
         if (ret < 0)
@@ -1436,6 +1600,8 @@ print_supportedCPUs (void)
     printf("\t%s\n",cascadelakeX_str);
     printf("\t%s\n",tigerlake_str);
     printf("\t%s\n",icelake_str);
+    printf("\t%s\n",rocketlake_str);
+    printf("\t%s\n",icelakesp_str);
     printf("\n");
     printf("Supported AMD processors:\n");
     printf("\t%s\n",opteron_sc_str);
