@@ -5,14 +5,14 @@
  *
  *      Description:  Interface to the procfs/sysfs based topology backend
  *
- *      Version:   5.1.0
- *      Released:  20.11.2020
+ *      Version:   5.2.2
+ *      Released:  26.07.2022
  *
  *      Authors:  Jan Treibig (jt), jan.treibig@gmail.com,
  *                Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2022 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -213,6 +213,12 @@ proc_init_cpuInfo(cpu_set_t cpuSet)
     const_bstring vendorString = bformat("CPU implementer\t:");
     const_bstring familyString = bformat("CPU architecture");
 #endif
+    cpuid_info.osname = malloc(MAX_MODEL_STRING_LENGTH * sizeof(char));
+    if (!cpuid_info.osname)
+    {
+        return;
+    }
+    cpuid_info.osname[0] = '\0';
 
     const_bstring countString = bformat("processor\t:");
 
@@ -226,8 +232,7 @@ proc_init_cpuInfo(cpu_set_t cpuSet)
 #endif
     cpuid_info.stepping = 0;
     cpuid_topology.numHWThreads = 0;
-    cpuid_info.osname = malloc(MAX_MODEL_STRING_LENGTH * sizeof(char));
-    cpuid_info.osname[0] = '\0';
+    
 
     if (NULL != (fp = fopen ("/proc/cpuinfo", "r")))
     {
@@ -249,18 +254,18 @@ proc_init_cpuInfo(cpu_set_t cpuSet)
                 cpuid_info.model = ownatoi(bdata(subtokens->entry[1]));
                 bstrListDestroy(subtokens);
 #else
-		const_bstring power7str = bformat("POWER7");
-		const_bstring power8str = bformat("POWER8");
-		const_bstring power9str = bformat("POWER9");
-		if (binstr(tokens->entry[i],0, power7str) != BSTR_ERR)
-		{
-			cpuid_info.model = POWER7;
-		}
-		else if (binstr(tokens->entry[i],0, power8str) != BSTR_ERR)
+                const_bstring power7str = bformat("POWER7");
+                const_bstring power8str = bformat("POWER8");
+                const_bstring power9str = bformat("POWER9");
+                if (binstr(tokens->entry[i],0, power7str) != BSTR_ERR)
+                {
+                    cpuid_info.model = POWER7;
+                }
+                else if (binstr(tokens->entry[i],0, power8str) != BSTR_ERR)
                 {
                         cpuid_info.model = POWER8;
                 }
-		else if (binstr(tokens->entry[i],0, power9str) != BSTR_ERR)
+                else if (binstr(tokens->entry[i],0, power9str) != BSTR_ERR)
                 {
                         cpuid_info.model = POWER9;
                 }
@@ -301,6 +306,9 @@ proc_init_cpuInfo(cpu_set_t cpuSet)
         }
         bstrListDestroy(tokens);
         cpuid_topology.numHWThreads = HWthreads;
+        HWthreads = likwid_sysfs_list_len("/sys/devices/system/cpu/present");
+        if (HWthreads > cpuid_topology.numHWThreads)
+            cpuid_topology.numHWThreads = HWthreads;
 #ifdef __x86_64
         snprintf(cpuid_info.architecture, 19, "x86_64");
 #endif
@@ -348,8 +356,8 @@ proc_init_cpuFeatures(void)
 #ifdef _ARCH_PPC
     return;
 #endif
-    const_bstring flagString = bformat("flags");
-    const_bstring featString = bformat("Features");
+    struct tagbstring flagString = bsStatic ("flags");
+    struct tagbstring featString = bsStatic ("Features");
     bstring flagline = bfromcstr("");
 
     bstring cpuinfo = read_file("/proc/cpuinfo");
@@ -358,14 +366,14 @@ proc_init_cpuFeatures(void)
     for (int i = 0; i < cpulines->qty; i++)
     {
 #if defined(__x86_64__) || defined(__i386__)
-        if (bstrncmp(cpulines->entry[i], flagString, 5) == BSTR_OK ||
-            bstrncmp(cpulines->entry[i], featString, 8) == BSTR_OK)
+        if (bstrncmp(cpulines->entry[i], &flagString, 5) == BSTR_OK ||
+            bstrncmp(cpulines->entry[i], &featString, 8) == BSTR_OK)
 #endif
 #if defined(__ARM_ARCH_7A__) || defined(__ARM_ARCH_8A__) || defined(__ARM_ARCH_8A)
-        if (bstrncmp(cpulines->entry[i], featString, 8) == BSTR_OK)
+        if (bstrncmp(cpulines->entry[i], &featString, 8) == BSTR_OK)
 #endif
 #ifdef _ARCH_PPC
-	if (ret != 1)
+    if (ret != 1)
 #endif
         {
             bdestroy(flagline);
@@ -376,6 +384,7 @@ proc_init_cpuFeatures(void)
     bstrListDestroy(cpulines);
 
     struct bstrList* flaglist = bsplit(flagline, ' ');
+    bdestroy(flagline);
     bstring bfeatures = bfromcstr("");
 
     cpuid_info.featureFlags = 0;
@@ -601,6 +610,7 @@ proc_init_nodeTopology(cpu_set_t cpuSet)
     int num_sockets = 0;
     int num_cores_per_socket = 0;
     int num_threads_per_core = 0;
+    int num_cores_per_die = 0;
 
     hwThreadPool = (HWThread*) malloc(cpuid_topology.numHWThreads * sizeof(HWThread));
     for (uint32_t i=0;i<cpuid_topology.numHWThreads;i++)
@@ -610,8 +620,9 @@ proc_init_nodeTopology(cpu_set_t cpuSet)
         hwThreadPool[i].threadId = -1;
         hwThreadPool[i].coreId = -1;
         hwThreadPool[i].packageId = -1;
+        hwThreadPool[i].dieId = -1;
         hwThreadPool[i].inCpuSet = 0;
-        if (CPU_ISSET(i, &cpuSet))
+        if (CPU_ISSET(i, &cpuSet) && likwid_cpu_online(i))
         {
             hwThreadPool[i].inCpuSet = 1;
         }
@@ -641,6 +652,18 @@ proc_init_nodeTopology(cpu_set_t cpuSet)
             fclose(fp);
         }
         bdestroy(file);
+        file = bformat("%s/die_id", bdata(cpudir));
+        if (NULL != (fp = fopen (bdata(file), "r")))
+        {
+            bstring src = bread ((bNread) fread, fp);
+            hwThreadPool[i].dieId = ownatoi(bdata(src));
+            if (hwThreadPool[i].packageId == 0 && hwThreadPool[i].dieId == 0)
+            {
+                num_cores_per_die++;
+            }
+            fclose(fp);
+        }
+        bdestroy(file);
         file = bformat("%s/thread_siblings_list", bdata(cpudir));
         if (NULL != (fp = fopen (bdata(file), "r")))
         {
@@ -656,20 +679,129 @@ proc_init_nodeTopology(cpu_set_t cpuSet)
         bdestroy(file);
 #ifdef __ARM_ARCH_8A
         if (hwThreadPool[i].packageId == -1)
+        {
             hwThreadPool[i].packageId = 0;
+            for (int j = 0; j < num_sockets*4; j++)
+            {
+                bstring nodestr = bformat("/sys/devices/system/cpu/cpu%d/node%d", i, j);
+                if (!access(bdata(nodestr), F_OK))
+                {
+                    hwThreadPool[i].packageId = j;
+                    int offset = 0;
+                    for (int k = 0; k < i; k++)
+                    {
+                        bstring teststr = bformat("/sys/devices/system/cpu/cpu%d/node%d/cpu%d", i, j, k);
+                        if (!access(bdata(teststr), F_OK))
+                            offset++;
+                        bdestroy(teststr);
+                    }
+                    if (hwThreadPool[i].coreId == -1)
+                        hwThreadPool[i].coreId = offset;
+                    break;
+                }
+                bdestroy(nodestr);
+            }
+        }
+        if (hwThreadPool[i].coreId == -1)
+            hwThreadPool[i].coreId = 0;
+        if (hwThreadPool[i].threadId == -1)
+            hwThreadPool[i].threadId = 0;
+        if (hwThreadPool[i].dieId == -1)
+            hwThreadPool[i].dieId = 0;
 #endif
-        DEBUG_PRINT(DEBUGLEV_DEVELOP, PROC Thread Pool PU %d Thread %d Core %d Socket %d inCpuSet %d,
+        DEBUG_PRINT(DEBUGLEV_DEVELOP, PROC Thread Pool PU %d Thread %d Core %d Die %d Socket %d inCpuSet %d,
                             hwThreadPool[i].apicId,
                             hwThreadPool[i].threadId,
                             hwThreadPool[i].coreId,
+                            hwThreadPool[i].dieId,
                             hwThreadPool[i].packageId,
                             hwThreadPool[i].inCpuSet)
         bdestroy(cpudir);
     }
+    int* helper = malloc(cpuid_topology.numHWThreads * sizeof(int));
+    if (!helper)
+    {
+        return;
+    }
     cpuid_topology.threadPool = hwThreadPool;
-    cpuid_topology.numSockets = num_sockets;
-    cpuid_topology.numCoresPerSocket = num_cores_per_socket;
+    int hidx = 0;
+    for (int i = 0; i < cpuid_topology.numHWThreads; i++)
+    {
+        int pid = hwThreadPool[i].packageId;
+        int found = 0;
+        for (int j = 0; j < hidx; j++)
+        {
+            if (pid == helper[j])
+            {
+                found = 1;
+                break;
+            }
+        }
+        if (!found)
+        {
+            helper[hidx++] = pid;
+        }
+    }
+    cpuid_topology.numSockets = hidx;
+    /* Traverse all sockets to get maximal thread count per socket.
+     * This should fix the code for architectures with "empty" sockets.
+     */
+    int num_threads_per_socket = 0;
+    for (int i = 0; i < cpuid_topology.numSockets; i++)
+    {
+        int threadCount = 0;
+        for (int j = 0; j < cpuid_topology.numHWThreads; j++)
+        {
+            if (helper[i] == hwThreadPool[j].packageId)
+            {
+                threadCount++;
+            }
+        }
+        if (threadCount > num_threads_per_socket)
+        {
+            num_threads_per_socket = threadCount;
+        }
+    }
+
+    int first_socket_id = helper[0];
+    hidx = 0;
+    for (int i = 0; i < cpuid_topology.numHWThreads; i++)
+    {
+        int did = hwThreadPool[i].dieId;
+        int pid = hwThreadPool[i].packageId;
+        if (pid != first_socket_id) continue;
+        int found = 0;
+        for (int j = 0; j < hidx; j++)
+        {
+            if (did == helper[j])
+            {
+                found = 1;
+                break;
+            }
+        }
+        if (!found)
+        {
+            helper[hidx++] = did;
+        }
+    }
+
+    cpuid_topology.numDies = hidx * cpuid_topology.numSockets;
+    if (cpuid_topology.numDies == cpuid_topology.numSockets)
+    {
+        cpuid_topology.numDies = 0;
+    }
+    int max_thread_sibling_id = 0;
+    for (int i = 0; i < cpuid_topology.numHWThreads; i++)
+    {
+        if (hwThreadPool[i].threadId > max_thread_sibling_id)
+        {
+            max_thread_sibling_id = hwThreadPool[i].threadId;
+        }
+    }
+    num_threads_per_core = max_thread_sibling_id + 1;
+    cpuid_topology.numCoresPerSocket = num_threads_per_socket/num_threads_per_core;
     cpuid_topology.numThreadsPerCore = num_threads_per_core;
+    free(helper);
     return;
 }
 
@@ -893,7 +1025,7 @@ proc_init_cacheTopology(void)
                 break;
             case ARMV8_FAMILY:
             case ARMV7_FAMILY:
-	    case PPC_FAMILY:
+            case PPC_FAMILY:
                 cachePool[i].inclusive = 0;
                 break;
             default:
