@@ -92,6 +92,7 @@ class EventParser():
                 .setName("pipe (|) separated list of subsystem names (PMC, PBOX0|PBOX1 etc.)"),
                 delim='|').setResultsName("subsystem")
             ).setParseAction(self._set_event)
+        self.Evt.ignore(Comment)
 
         self.Options = (
             LinePrefix("OPTIONS") + EventName
@@ -104,6 +105,7 @@ class EventParser():
                 .setName("pipe (|) separated list of option names")
             - LineEnd()
             ).setParseAction(self._set_options)
+        self.Options.ignore(Comment)
 
         self.DefaultOptions = (
             LinePrefix("DEFAULT_OPTIONS") + EventName
@@ -119,6 +121,7 @@ class EventParser():
                 .setName("option list (comma separated list of NAME=VALUE pairs)")
             - LineEnd()
             ).setParseAction(self._set_default_options)
+        self.DefaultOptions.ignore(Comment)
 
         self.Umask = (
             LinePrefix("UMASK") + EventName
@@ -127,6 +130,7 @@ class EventParser():
                 .setName("UMASK value (integer)")
             - LineEnd()
             ).setParseAction(self._parse_umask)
+        self.Umask.ignore(Comment)
 
         self.Event = ( self.Umask | self.Evt | self.Options | self.DefaultOptions | Comment ) + StringEnd()
 
@@ -512,7 +516,8 @@ class GroupParser():
         long = Keyword('LONG').suppress() + EOL
         self.parser += long
 
-        formulae = Keyword('Formulas:') + EOL + OneOrMore(Line().setParseAction(self.add_formula))
+        Formula = LineStart() + SkipTo(LineEnd(), failOn=LineStart()+(Keyword('-') ^ Keyword('--'))+LineEnd()) + EOL
+        formulae = Keyword('Formulas:') + EOL + OneOrMore(Formula().setParseAction(self.add_formula))
         self.parser += formulae
 
         descr = (Keyword('-') ^ Keyword('--')).suppress() + EOL + Group(OneOrMore(Line()))
@@ -541,7 +546,11 @@ class GroupParser():
 
     def add_formula(self, s, loc, toks):
         if not '=' in toks[0]:
-            raise ParseException(s,loc,"expected '=' sign")
+            i = [toks[0].find(x['event']) for x in self.group['events'] if toks[0].find(x['event'])>=0]
+            pos = min(i) if i else 0
+            while pos > 0 and toks[0][pos] != ' ':
+                pos -= 1
+            raise ParseSyntaxException(s,loc + pos, "expected '=' sign")
         metric, formula = [ x.strip() for x in toks[0].split('=', 1) ]
         dim = miss_brackets_in_units(metric)
         if dim:
@@ -652,6 +661,9 @@ def start_logging(args):
 
 
 
+def abs_path(path):
+    return Path(path).expanduser().resolve()
+
 if __name__ == "__main__":
 
     import argparse
@@ -669,9 +681,9 @@ if __name__ == "__main__":
     etest.add_argument('--verbose', '-v', action='store_true', default=False,
                        help="more verbose output"
                        )
-    default = Path(__file__).parent / "../src/includes"
+    default = abs_path(__file__).parent / "../src/includes"
     default = default.resolve()
-    etest.add_argument('--input-dir', '-d', type=Path, default=default,
+    etest.add_argument('--input-dir', '-d', type=abs_path, default=default,
                 help="path to the directory with event data files, "
                      "default: {}".format(default)
                 )
@@ -684,7 +696,7 @@ if __name__ == "__main__":
     etest.add_argument('--json', '-j', action='store_true', default=False,
                 help="dump event descriptions in JSON format"
                 )
-    etest.add_argument('--output-dir', '-o', type=Path, default=Path('.'),
+    etest.add_argument('--output-dir', '-o', type=abs_path, default=abs_path('.'),
                 help="directory where to put output JSON files, "
                      "default: current directory"
                 )
@@ -700,20 +712,20 @@ if __name__ == "__main__":
     gtest = subparsers.add_parser('groups', help="test group ddescriptions")
     gtest.set_defaults(func=check_groups)
     gtest.add_argument('--verbose', '-v', action='count', default=0)
-    default = Path(__file__).parent / "../groups"
+    default = abs_path(__file__).parent / "../groups"
     if default.exists():
         default = default.resolve()
     else:
-        default = Path(__file__).parent
-    gtest.add_argument('--input-dir', '-d', type=Path,
+        default = abs_path(__file__).parent
+    gtest.add_argument('--input-dir', '-d', type=abs_path,
                         default=default,
                         help='path to the directory with groups data files, default: {}'.format(default))
     gtest.add_argument('files', metavar='FILE', type=str, nargs='*',
                     help='group data file to check')
     gtest.add_argument('--json', '-j', action='store_true', default=False,
                     help="dump group descriptions in JSON format")
-    gtest.add_argument('--output-dir', '-o', type=Path,
-                        default=Path('.'),
+    gtest.add_argument('--output-dir', '-o', type=abs_path,
+                        default=abs_path('.'),
                         help='directory where to put output JSON files, default: current directory')
 
     args = ap.parse_args()
